'''
Code to separate Equations into independent subsets. This can be used in
STDP to allow for nonlinear equations to be given. Also, a modified version of
this code could probably be used to define independent equations for use in
compound solvers. For example, you could split into separate independent sets
and have one solver for each set. In some circumstances, this might be more
efficient (e.g. a linear and nonlinear component). It may need to be modified so
that the dependencies are directional, i.e. X depending on Y doesn't mean Y
depends on X (which you want in the case of the STDP separation), that way you
could have a linear solver for a subset and a nonlinear solver that depends on
some of the linear variables. In this case you would probably look for minimal
subgraphs which are allowed to have incoming edges but no outgoing edges,
construct solvers for them, and then construct a compound solver for the rest.
'''

if __name__ == '__main__':
    from brian.equations import Equations
    from brian.inspection import get_identifiers
else:
    from ..equations import Equations
    from ..inspection import get_identifiers
from collections import defaultdict
from copy import copy

__all__ = ['separate_equations']

def next_independent_subgraph(G):
    '''
    Returns an independent subgraph of G and removes that subgraph from G.
    
    Here G is a graph represented as a dictionary, so that G[node] is a set
    of all the neighbours of node. The caller is responsible for constructing
    a well formed G which has all the appropriate nodes, and where edges in
    the graph are both ways so that b in G[a] <=> a in G[b].
    '''
    # We start from just one (randomly selected) node and iteratively add
    # elements to the subgraph from the set of neighbours of the subgraph
    # until there are no more neighbours to be added, and then we return.
    # This algorithm may not be efficient but we're likely only dealing with
    # fairly small graphs anyway.
    nodes = dict([G.popitem()])
    found_new = True
    while found_new:
        found_new = False
        for node, targets in nodes.items():
            for target in targets:
                if target in G: # if target not in G, we assume it has already been added
                    nodes[target] = G.pop(target)
                    found_new = True
                else: # this should be true if the caller has constructed G correctly
                    assert target in nodes
    return nodes

def separate_equations(eqs, additional_dependencies=[]):
    eqs.prepare()
    all_vars = eqs._string.keys()
    # Construct a list of dependency sets, where each variable in each
    # dependency set induces a dependency on each other variable in each
    # dependency set
    depsets = []
    for var in all_vars:
        ids = set(get_identifiers(eqs._string[var]))
        ids = ids.intersection(all_vars)
        ids.add(var)
        depsets.append(ids)
    for expr in additional_dependencies:
        ids = set(get_identifiers(expr))
        ids = ids.intersection(all_vars)
        depsets.append(ids)
    # Construct a graph deps which indicates what variable depends on which
    # other variables (or is depended on by other variables).
    deps = defaultdict(set)
    for ids in depsets:
        for id1 in ids:
            for id2 in ids:
                deps[id1].add(id2)
    # Extract all the independent subgraphs
    ind_graphs = []
    while len(deps):
        ind_graphs.append(set(next_independent_subgraph(deps).keys()))
    if len(ind_graphs) == 1:
        return [eqs]
    # Finally, we construct an Equations object for each of the subgraphs
    ind_eqs = []
    for G in ind_graphs:
        neweqs = Equations()
        for var in G:
            if var in eqs._eq_names:
                neweqs.add_eq(var, eqs._string[var], eqs._units[var],
                              local_namespace=eqs._namespace[var])
            elif var in eqs._diffeq_names:
                nonzero = var in eqs._diffeq_names_nonzero
                neweqs.add_diffeq(var, eqs._string[var], eqs._units[var],
                                  local_namespace=eqs._namespace[var],
                                  nonzero=nonzero)
            elif var in eqs._alias.keys():
                neweqs.add_alias(var, eqs._string[var].strip())
            else:
                assert False
        ind_eqs.append(neweqs)
    return ind_eqs

if __name__ == '__main__':
    from brian import *
#    T = second
#    eqs = Equations('''
#    da/dt = y/T : 1
#    db/dt = x/T : 1
#    c : 1
#    x = c
#    y = x*x : 1
#    dd/dt = e/T : 1
#    de/dt = d/T : 1
#    df/dt = e/T : 1
#    ''')

    tau_pre = tau_post = 10 * ms
    eqs = Equations("""
    dA_pre/dt  = -A_pre/tau_pre   : 1
    dA_post/dt = -A_post/tau_post : 1
    """)

    eqs_sep = separate_equations(eqs)
    for e in eqs_sep:
        print '******************'
        print e



from numpy.random import *
from pycuda.autoinit import *
from pycuda import *
#from pycuda.gpuarray import *
from pycuda import driver as drv
from ctypes import *
from numpy import *

lib = cdll.LoadLibrary('testchagpp')
find_positive = lib.find_positive
print find_positive

x = randn(100)
I = (x > 0).nonzero()[0]

y = drv.mem_alloc(100 * 8)
drv.memcpy_htod(y, x)

out = drv.mem_alloc(100 * 8)
count = drv.mem_alloc(4)

#y = to_gpu(x)
#out = to_gpu(zeros(len(x)))
#count = to_gpu(array([0], dtype=int))

find_positive(int(y), int(y) + len(x) * 8, int(out), int(count))

#find_positive(int(y.gpudata), int(y.gpudata)+len(x)*8,
#              int(out.gpudata), int(count.gpudata))

z = zeros(100)
drv.memcpy_dtoh(z, out)

c = array([0], dtype=int)
drv.memcpy_dtoh(c, count)

z = z[:c[0]]

#z = out.get()[:count.get()[0]]

print z - x[I]



'''
This example demonstrates how to load AER data files and use them as a
SpikeGeneratorGroup in Brian


For more information either check the documentation in the package,
post to the google group, or send me an email victor.benichoux@ens.fr

'''
from brian import *
from brian.experimental.neuromorphic import *


filename = '/path/to/file' # support .aedat and .aeidx files

addr, timestamps = load_AER(filename) # load the data
# addr contains a list of addresses
# timestamps the list of spike times (the first spike comes in at t = 0, but you may change that
# Note: In the case of an aeidx file, the value returned is a list of tuples (addr, timestamp)

group = AERSpikeGeneratorGroup((addr, timestamps))# Create an AER group

# at that point you can do whatever you want, it is a regular Brian neuron group

# * Events addressing * 
#
# We provide with two event extraction functions that fetch the
# specialized addresses of the DVS retina and the AMS cochlea.
# For example:
(x, y, pol) = extract_DVS_event(addr) 
# returns the x and y positions of the neuron, alongside the polarity
# (on/off) of the event
# See the documentation for more on this.

M = SpikeMonitor(group)# monitor it

run(group.maxtime) # this group has an additional attribute maxtime
# that gives you the last spike time

# plot it
raster_plot(M)
show()

# Additionally you can save any SpikeMonitor to and aedat file format as follows
save(M, './dummy.aedat')



# AEC experiment (voltage-clamp)
from brian import *
from brian.library.electrophysiology import *

myclock = Clock(dt=.1 * ms)
clock_rec = Clock(dt=.1 * ms)

#log_level_debug()

taum = 20 * ms
gl = 20 * nS
Cm = taum * gl
Re = 50 * Mohm
Ce = 0.1 * ms / Re

eqs = Equations('''
dvm/dt=(-gl*vm+i_inj)/Cm : volt
I:amp
''')
eqs += electrode(.6 * Re, Ce)
eqs += current_clamp(vm='v_el', i_inj='i_cmd', i_cmd='I', Re=.4 * Re, Ce=Ce)
setup = NeuronGroup(1, model=eqs, clock=myclock)
board = VC_AEC(setup, 'v_rec', 'I', gain=300 * nS, gain2=20 * nS / ms, clock=clock_rec)
recording = StateMonitor(board, 'record', record=True, clock=myclock)
soma = StateMonitor(setup, 'vm', record=True, clock=myclock)

run(50 * ms)
board.start_injection()
run(1 * second)
board.stop_injection()
run(100 * ms)
board.estimate()
print 'Re=', sum(board.Ke) * ohm
board.switch_on()
run(50 * ms)
board.command = 10 * mV
run(200 * ms)
board.command = 0 * mV
run(150 * ms)
board.switch_off()
figure()
plot(recording.times / nA, recording[0] / nA, 'b')
figure()
plot(soma.times / ms, soma[0] / mV, 'r')
show()




'''
'''
# TODO: support for GPUBufferedArray?
from numpy import *
import pycuda
#import pycuda.autoinit as autoinit
import pycuda.driver as drv
import pycuda.compiler
from pycuda import gpuarray
from brian.experimental.cuda.buffering import *
import re
from filterbank import Filterbank, RestructureFilterbank
from gputools import *
import gc

__all__ = ['LinearFilterbank']

class LinearFilterbank(Filterbank):
    '''
    Generalised parallel linear filterbank
    
    This filterbank allows you to construct a chain of linear filters in
    a bank so that each channel in the bank has its own filters. You pass
    the (b,a) parameters for the filter in the format specified in the
    function ``apply_linear_filterbank``.
    
    Note that there are additional GPU specific options:
    
    ``precision``
        Should be single for older GPUs.
    ``forcesync``
        Should be set to true to force a copy of GPU->CPU after each
        filter step, but not if the output is being used in ongoing
        GPU computations. By default this is ``True`` for compatibility.
    ``pagelocked_mem``
        Allocate faster pagelocked memory for GPU->CPU copies (doubles
        copy rate).
    ``unroll_filterorder``
        Whether or not to unroll the loop for the filter order, normally
        this should be done for small filter orders. By default, it is done
        if the filter order is less than or equal to 32.
    '''
    def __init__(self, source, b, a, samplerate=None,
                 precision='double', forcesync=True, pagelocked_mem=True, unroll_filterorder=None):
        # Automatically duplicate mono input to fit the desired output shape
        if b.shape[0]!=source.nchannels:
            if source.nchannels!=1:
                raise ValueError('Can only automatically duplicate source channels for mono sources, use RestructureFilterbank.')
            source = RestructureFilterbank(source, b.shape[0])
        Filterbank.__init__(self, source)
        if pycuda.context is None:
            set_gpu_device(0)
        self.precision=precision
        if self.precision=='double':
            self.precision_dtype=float64
        else:
            self.precision_dtype=float32
        self.forcesync=forcesync
        self.pagelocked_mem=pagelocked_mem
        n, m, p=b.shape
        self.filt_b=b
        self.filt_a=a
        filt_b_gpu=array(b, dtype=self.precision_dtype)
        filt_a_gpu=array(a, dtype=self.precision_dtype)
        filt_state=zeros((n, m-1, p), dtype=self.precision_dtype)
        if pagelocked_mem:
            filt_y=drv.pagelocked_zeros((n,), dtype=self.precision_dtype)
            self.pre_x=drv.pagelocked_zeros((n,), dtype=self.precision_dtype)
        else:
            filt_y=zeros(n, dtype=self.precision_dtype)
            self.pre_x=zeros(n, dtype=self.precision_dtype)
        self.filt_b_gpu=gpuarray.to_gpu(filt_b_gpu.T.flatten()) # transform to Fortran order for better GPU mem
        self.filt_a_gpu=gpuarray.to_gpu(filt_a_gpu.T.flatten()) # access speeds
        self.filt_state=gpuarray.to_gpu(filt_state.T.flatten())
        self.unroll_filterorder = unroll_filterorder
        if unroll_filterorder is None:
            if m<=32:
                unroll_filterorder = True
            else:
                unroll_filterorder = False
        # TODO: improve code, check memory access patterns, maybe use local memory
        code='''
        #define x(s,i) _x[(s)*n+(i)]
        #define y(s,i) _y[(s)*n+(i)]
        #define a(i,j,k) _a[(i)+(j)*n+(k)*n*m]
        #define b(i,j,k) _b[(i)+(j)*n+(k)*n*m]
        #define zi(i,j,k) _zi[(i)+(j)*n+(k)*n*(m-1)]
        __global__ void filt(SCALAR *_b, SCALAR *_a, SCALAR *_x, SCALAR *_zi, SCALAR *_y, int numsamples)
        {
            int j = blockIdx.x * blockDim.x + threadIdx.x;
            if(j>=n) return;
            for(int s=0; s<numsamples; s++)
            {
        '''
        for k in range(p):
            loopcode='''
            y(s,j) = b(j,0,k)*x(s,j) + zi(j,0,k);
            '''
            if unroll_filterorder:
                for i in range(m-2):
                    loopcode+=re.sub('\\bi\\b', str(i), '''
                    zi(j,i,k) = b(j,i+1,k)*x(s,j) + zi(j,i+1,k) - a(j,i+1,k)*y(s,j);
                    ''')
            else:
                loopcode+='''
                for(int i=0;i<m-2;i++)
                    zi(j,i,k) = b(j,i+1,k)*x(s,j) + zi(j,i+1,k) - a(j,i+1,k)*y(s,j);
                '''
            loopcode+='''
            zi(j,m-2,k) = b(j,m-1,k)*x(s,j) - a(j,m-1,k)*y(s,j);
            '''
            if k<p-1:
                loopcode+='''
                x(s,j) = y(s,j);
                '''
            loopcode=re.sub('\\bk\\b', str(k), loopcode)
            code+=loopcode
        code+='''
            }
        }
        '''
        code=code.replace('SCALAR', self.precision)
        code=re.sub("\\bp\\b", str(p), code) #replace the variable by their values
        code=re.sub("\\bm\\b", str(m), code)
        code=re.sub("\\bn\\b", str(n), code)
        #print code
        self.gpu_mod=pycuda.compiler.SourceModule(code)
        self.gpu_filt_func=self.gpu_mod.get_function("filt")
        blocksize=256
        if n<blocksize:
            blocksize=n
        if n%blocksize==0:
            gridsize=n/blocksize
        else:
            gridsize=n/blocksize+1
        self.block=(blocksize, 1, 1)
        self.grid=(gridsize, 1)
        self.gpu_filt_func.prepare((intp, intp, intp, intp, intp, int32), self.block)
        self._has_run_once=False

    def reset(self):
        self.buffer_init()

    def buffer_init(self):
        Filterbank.buffer_init(self)
        self.filt_state.set(zeros(self.filt_state.shape, dtype=self.filt_state.dtype))
    
    def buffer_apply(self, input):
        # TODO: buffer apply to a large input may cause a launch timeout, need to buffer in
        # smaller chunks if this is the case
        b = self.filt_b_gpu
        a = self.filt_a_gpu
        zi = self.filt_state
        if not hasattr(self, 'filt_x_gpu') or input.size!=self.filt_x_gpu.size:
            self._desiredshape = input.shape
            self._has_run_once = False
            self.filt_x_gpu = gpuarray.to_gpu(input.flatten())
            self.filt_y_gpu = gpuarray.empty_like(self.filt_x_gpu)
        else:
            self.filt_x_gpu.set(input.flatten())
        filt_x_gpu = self.filt_x_gpu
        filt_y_gpu = self.filt_y_gpu
        if self._has_run_once:
            self.gpu_filt_func.launch_grid(*self.grid)
        else:
            self.gpu_filt_func.prepared_call(self.grid, intp(b.gpudata),
                    intp(a.gpudata), intp(filt_x_gpu.gpudata),
                    intp(zi.gpudata), intp(filt_y_gpu.gpudata), int32(input.shape[0]))
            self._has_run_once = True
        return reshape(filt_y_gpu.get(pagelocked=self.pagelocked_mem), self._desiredshape)

    def decascade(self, ncascade=1):
        '''
        Reduces cascades of low order filters into smaller cascades of high order filters.
        
        ``ncascade`` is the number of cascaded filters to use, which should be
        a divisor of the original number.
        
        Note that higher order filters are often numerically unstable.
        '''
        n, m, p = self.filt_b.shape
        if p%ncascade!=0:
            raise ValueError('Number of cascades must be a divisor of original number of cascaded filters.')
        b = zeros((n, (m-1)*(p/ncascade)+1, ncascade))
        a = zeros((n, (m-1)*(p/ncascade)+1, ncascade))
        for i in xrange(n):
            for k in xrange(ncascade):
                bp = ones(1)
                ap = ones(1)
                for j in xrange(k*(p/ncascade), (k+1)*(p/ncascade)):
                    bp = polymul(bp, self.filt_b[i, ::-1, j])
                    ap = polymul(ap, self.filt_a[i, ::-1, j])
                bp = bp[::-1]
                ap = ap[::-1]
                a0 = ap[0]
                ap /= a0
                bp /= a0
                b[i, :len(bp), k] = bp
                a[i, :len(ap), k] = ap
        LinearFilterbank.__init__(self, self.source, b, a, samplerate=self.samplerate,
                      precision=self.precision, forcesync=self.forcesync,
                      pagelocked_mem=self.pagelocked_mem,
                      unroll_filterorder=self.unroll_filterorder)



'''
Proof of concept - CUBA on a cluster with MPI

Slower than on a single processor but it works!
'''
from brian import *
import pypar
import time


class ReceiverThreshold(Threshold):
    '''
    Receives spikes from other processor
    '''
    def __init__(self, owner):
        self.owner = owner # processor owning the group

    def __call__(self, P):
        nspikes = array([0])
        pypar.broadcast(nspikes, self.owner)
        nspikes = nspikes[0]
        spikes = zeros(nspikes, dtype=int)
        if nspikes > 0:
            pypar.broadcast(spikes, self.owner)
        return spikes


class SenderThreshold(Threshold):
    '''
    Sends spikes to other processors
    '''
    def __init__(self, owner, threshold):
        self.owner = owner # processor owning the group
        self._threshold = threshold

    def __call__(self, P):
        spikes = array(self._threshold(P), dtype=int)
        nspikes = array([len(spikes)])
        pypar.broadcast(nspikes, self.owner)
        nspikes = nspikes[0]
        if nspikes > 0:
            pypar.broadcast(spikes, self.owner)
        return spikes

# Identification
myid = pypar.rank() # id of this process
nproc = pypar.size() # number of processors
node = pypar.get_processor_name()

print "I am processor %d of %d on node %s" % (myid, nproc, node)

if nproc != 2:
    raise Exception, "This example only works with 2 processors"

N = 32000
Ne = int(N * .8)
Ni = int(N * .2)
sparseness = 80. / N
taum = 20 * ms
taue = 5 * ms
taui = 10 * ms
Vt = -50 * mV
Vr = -60 * mV
El = -49 * mV
we = (60 * 0.27 / 10) * mV # excitatory synaptic weight (voltage)
wi = (-20 * 4.5 / 10) * mV # inhibitory synaptic weight

eqs = Equations('''
dv/dt  = (ge+gi-(v-El))/taum : volt
dge/dt = -ge/taue : volt
dgi/dt = -gi/taui : volt
''')

if myid == 0: # excitatory group
    Pe = NeuronGroup(Ne, model=eqs, threshold=Vt, reset=Vr, refractory=5 * ms)
    Pe._threshold = SenderThreshold(0, Pe._threshold)
    Pi = NeuronGroup(Ni, model=eqs, threshold=ReceiverThreshold(1))
    Pi._state_updater = LazyStateUpdater()
    Pe.v = Vr + rand(len(Pe)) * (Vt - Vr)
    Cee = Connection(Pe, Pe, 'ge', weight=we, sparseness=sparseness)
    Cie = Connection(Pi, Pe, 'gi', weight=wi, sparseness=sparseness)

else: # inhibitory group
    Pi = NeuronGroup(Ni, model=eqs, threshold=Vt, reset=Vr, refractory=5 * ms)
    Pi._threshold = SenderThreshold(1, Pi._threshold)
    Pe = NeuronGroup(Ne, model=eqs, threshold=ReceiverThreshold(0))
    Pe._state_updater = LazyStateUpdater()
    Pi.v = Vr + rand(len(Pi)) * (Vt - Vr)
    Cei = Connection(Pe, Pi, 'ge', weight=we, sparseness=sparseness)
    Cii = Connection(Pi, Pi, 'gi', weight=wi, sparseness=sparseness)

# Record the number of spikes
Me = PopulationSpikeCounter(Pe)
Mi = PopulationSpikeCounter(Pi)
# A population rate monitor
#M = PopulationRateMonitor(P)

print len(Pe) + len(Pi), "neurons in the network"
print "Simulation running..."
start_time = time.time()

run(1 * second)

duration = time.time() - start_time
print "Simulation time:", duration, "seconds"
print Me.nspikes, "excitatory spikes"
print Mi.nspikes, "inhibitory spikes"
#plot(M.times/ms,M.smooth_rate(2*ms,'gaussian'))
#show()

pypar.finalize()



from brian import *
from matplotlib.widgets import Button

fig = figure()
plot([0, 0], [1, 1])

def onbutton(e):
    x, y = e.xdata, e.ydata
    print x, y

def nextbutton(e):
    print 'hello'

fig.canvas.mpl_connect('button_press_event', onbutton)

axnext = axes([0.81, 0.05, 0.1, 0.075])
bnext = Button(axnext, 'Next')
bnext.on_clicked(nextbutton)

show()



'''
Voltage-clamp experiment
'''
from brian import *
from brian.library.electrophysiology import *

defaultclock.dt = .01 * ms

taum = 20 * ms
gl = 20 * nS
Cm = taum * gl
Re = 50 * Mohm
Ce = 0.2 * ms / Re
N = 1
Rs = .9 * Re
tauc = Rs * Ce # critical tau_u

eqs = Equations('''
dvm/dt=(-gl*vm+i_inj)/Cm : volt
''')
eqs += electrode(.2 * Re, Ce)
eqs += voltage_clamp(vm='v_el', v_cmd=20 * mV, i_inj='i_cmd', i_rec='ic',
                   Re=.8 * Re, Rs=.9 * Re, tau_u=.2 * ms)
setup = NeuronGroup(N, model=eqs)
setup.v = 0 * mV
recording = StateMonitor(setup, 'ic', record=True)
soma = StateMonitor(setup, 'vm', record=True)

run(200 * ms)
figure()
plot(recording.times / ms, recording[0] / nA, 'k')
figure()
plot(soma.times / ms, soma[0] / mV, 'b')
show()



from brian import *

def test_structures():
    reinit_default_clock()
    H = NeuronGroup(1, 'V:1\nmod:1', reset=0, threshold=1)
    H.V = 2
    H.mod = 1
    G = [NeuronGroup(10, 'V:1', reset=0, threshold=1) for _ in range(12)]
    M = [SpikeMonitor(g) for g in G]
    C0 = Connection(H, G[0], 'V', weight=2)
    C1 = Connection(H, G[1], 'V', weight=2, structure='dense')
    C2 = Connection(H, G[2], 'V', weight=2, structure='dynamic')
    C3 = Connection(H, G[3], 'V', weight=2, modulation='mod')
    C4 = Connection(H, G[4], 'V', weight=2, structure='dense', modulation='mod')
    C5 = Connection(H, G[5], 'V', weight=2, structure='dynamic', modulation='mod')
    C6 = Connection(H, G[6], 'V', weight=2, delay=True)
    C7 = Connection(H, G[7], 'V', weight=2, delay=True, structure='dense')
    C8 = Connection(H, G[8], 'V', weight=2, delay=True, structure='dynamic')
    C9 = Connection(H, G[9], 'V', weight=2, delay=True, modulation='mod')
    C10 = Connection(H, G[10], 'V', weight=2, delay=True, structure='dense', modulation='mod')
    C11 = Connection(H, G[11], 'V', weight=2, delay=True, structure='dynamic', modulation='mod')
    for c in [C6, C7, C8, C9, C10, C11]:
        c.delay[0, :] = arange(10) * defaultclock.dt + defaultclock.dt / 2
    run(2 * ms)
    for k, m in enumerate(M):
        assert len(m.spikes), 'Problem with connection ' + str(k)
        i, j = zip(*m.spikes)
        i = array(i)
        j = array(j)
        j = array((j + defaultclock.dt / 2) / defaultclock.dt, dtype=int)
        assert (i == arange(10)).all(), 'Problem with connection ' + str(k)
        if k < 6:
            assert (j == 1).all(), 'Problem with connection ' + str(k)
        else:
            assert (j == (1 + arange(10))).all(), 'Problem with connection ' + str(k) + ': j=' + str(j)

if __name__ == '__main__':
    test_structures()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Generation of correlated spike trains.
Based on the article: Brette, R. (2009). Generation of correlated spike trains.
http://audition.ens.fr/brette/papers/Brette2008NC.html

The models for correlated spike trains are from the paper
but the implemented algorithms are simple ones.

See brian.utils.statistics for correlograms.
'''
from ..threshold import PoissonThreshold, HomogeneousPoissonThreshold
from ..neurongroup import NeuronGroup
from ..equations import Equations
from scipy.special import erf
from scipy.optimize import newton, fmin_tnc
from scipy import *
from ..units import check_units, hertz, second
from ..utils.circular import SpikeContainer
from numpy.random import poisson, binomial, rand, exponential
from random import sample

__all__ = ['rectified_gaussian', 'inv_rectified_gaussian', 'HomogeneousCorrelatedSpikeTrains', \
         'MixtureHomogeneousCorrelatedSpikeTrains',
         'CorrelatedSpikeTrains', 'mixture_process', 'find_mixture']

"""
Utility functions
"""
def rectified_gaussian(mu, sigma):
    '''
    Calculates the mean and standard deviation for a rectified Gaussian distribution.
    mu, sigma: parameters of the original distribution
    Returns mur,sigmar: parameters of the rectified distribution
    '''
    a = 1. + erf(mu / (sigma * (2 ** .5)));

    mur = (sigma / (2. * pi) ** .5) * exp(-0.5 * (mu / sigma) ** 2) + .5 * mu * a
    sigmar = ((mu - mur) * mur + .5 * sigma ** 2 * a) ** .5

    return (mur, sigmar)

def inv_rectified_gaussian(mur, sigmar):
    '''
    Inverse of the function rectified_gaussian
    '''
    if sigmar == 0 * sigmar: # for unit consistency
        return (mur, sigmar)

    x0 = mur / sigmar
    ratio = lambda u, v:u / v
    f = lambda x:ratio(*rectified_gaussian(x, 1.)) - x0
    y = newton(f, x0 * 1.1) # Secant method
    sigma = mur / (exp(-0.5 * y ** 2) / ((2. * pi) ** .5) + .5 * y * (1. + erf(y * (2 ** (-.5)))))
    mu = y * sigma

    return (mu, sigma)

"""
Doubly stochastic processes (Cox processes).
Good for long correlation time constants.
"""

class HomogeneousCorrelatedSpikeTrains(NeuronGroup):
    '''
    Correlated spike trains with identical rates and homogeneous exponential correlations.
    Uses Cox processes (Ornstein-Uhlenbeck).
    '''
    @check_units(r=hertz, tauc=second)
    def __init__(self, N, r, c, tauc, clock=None):
        '''
        Initialization:
        r = rate (Hz)
        c = total correlation strength (in [0,1])
        tauc = correlation time constant (ms)
        Cross-covariance functions are (c*r/tauc)*exp(-|s|/tauc)
        '''
        self.N = N
        # Correction of mu and sigma
        sigmar = (c * r / (2. * tauc)) ** .5
        mu, sigma = inv_rectified_gaussian(r, sigmar)
        eq = Equations('drate/dt=(mu-rate)/tauc + sigma*xi/tauc**.5 : Hz', mu=mu, tauc=tauc, sigma=sigma)
        NeuronGroup.__init__(self, 1, model=eq, threshold=HomogeneousPoissonThreshold(),
                             clock=clock)
        self._use_next_allowed_spiketime_refractoriness = False
        self.rate = mu
        self.LS = SpikeContainer(1) # Spike storage

    def __len__(self):
        # We need to redefine this because it is not the size of the state matrix
        return self.N

    def __getslice__(self, i, j):
        Q = NeuronGroup.__getslice__(self, i, j) # Is this correct?
        Q.N = j - i
        return Q



class MixtureHomogeneousCorrelatedSpikeTrains(NeuronGroup):
    def __init__(self, N, r, c, cl=None):
        """
        Generates a pool of N homogeneous correlated spike trains with identical
        rates r and correlation c (0 <= c <= 1).
        """
        NeuronGroup.__init__(self, N, model="v : 1")
        self.N = N
        self.r = r
        self.c = c
        if cl is None:
            cl = guessclock()
        self.clock = cl
        self.dt = cl.dt

    # called at each time step, returns the spiking neurons?
    def update(self):
        # Generates a source Poisson spike train with rate r/c
        if rand(1) <= self.r / self.c * self.dt:
            # If there is a source spike, it is copied to each target train with probability c
            spikes = nonzero(rand(self.N) <= self.c)[0]
        else:
            spikes = []
        self.LS.push(spikes)



def decompose_correlation_matrix(C, R):
    '''
    Completes the diagonal of C and finds L such that C=LL^T.
    C is matrix of correlation coefficients with unspecified diagonal.
    R is the rate vector.
    C must be symmetric.
    N.B.: The diagonal of C is modified (with zeros).
    '''
    # 0) Remove diagonal entries and calculate D (var_i(x) is should have magnitude r_i^2)
    D = R ** 2
    C -= diag(diag(C))

    # Completion
    # 1) Calculate D^{-1}C
    L = dot(diag(1. / D), C)

    # 2) Find the smallest eigenvalue
    eigenvals = linalg.eig(L)[0]
    alpha = -min(eigenvals[isreal(eigenvals)])

    # 3) Complete the diagonal with alpha*ri^2
    #alpha=alpha+.01; // avoids bad conditioning problems (uncomment if Cholesky fails)
    C += diag(alpha * D)

    # 4) Calculate a square root (Cholesky is unstable, use singular value decomposition)
    #return linalg.cholesky(C,lower=1)
    U, S, V = linalg.svd(C)
    return dot(dot(U, sqrt(diag(S))), V.T)


class CorrelatedSpikeTrains(NeuronGroup):
    '''
    Correlated spike trains with arbitrary rates and pair-wise exponential correlations.
    Uses Cox processes (Ornstein-Uhlenbeck).
    P.rate is the vector of (time-varying) rates.
    '''
    @check_units(tauc=second)
    def __init__(self, rates, C, tauc, clock=None):
        '''
        Initialization:
        rates = rates (Hz)
        C = correlation matrix
        tauc = correlation time constant (ms)
        Cross-covariance functions are C[i,j]*exp(-|s|/tauc)
        '''
        eq = Equations('''
        rate : Hz
        dy/dt = -y*(1./tauc)+xi/(.5*tauc)**.5 : 1
        ''')
        NeuronGroup.__init__(self, len(rates), model=eq, threshold=PoissonThreshold(), \
                             clock=clock)
        self._R = array(rates)
        self._L = decompose_correlation_matrix(array(C), self._R)

    def update(self):
        # Calculate rates
        self.rate_ = self._R + dot(self._L, self.y_)
        NeuronGroup.update(self)

row_vector = lambda V:V.reshape(1, len(V))
column_vector = lambda V:V.reshape(len(V), 1)

def homogeneous_mixture(R, c):
    '''
    Returns a mixture (nu,P) for a homogeneous synchronization structure:
    C(i,j)=2*c*(R(i)*R(j))/<R>
    '''
    pass

def find_mixture(R, C, iter=10000):
    '''
    Finds a mixture matrix P and source rate vector nu
    from the correlation matrix C and the target rates R.
    Returns nu,P.
    
    Gradient descent.
    TODO: use Scipy optimization algorithms.
    '''
    N = len(R)

    # Steps
    b = 0.1 / N
    a = b / N

    # Initial value: here such that F=0
    P = eye(N)
    R = column_vector(R)
    nu = row_vector(R)

    for _ in xrange(iter):
        # Compute error
        Q = dot(P, diag(sqrt(nu.flatten())))
        C2 = dot(Q, Q.T)
        E = linalg.norm(C - C2 - diag(diag(C - C2)), 'fro')
        F = sum(clip(dot(P, nu.T) - R, 0, Inf))
        A = -(C - C2 - diag(diag(C - C2)))

        #print "E=",E,"F=",F,"Etot=",E*a+F*b

        # Gradient in E
        dPE = 4 * dot(dot(A, P), diag(nu.flatten()))
        dNuE = row_vector(2 * diag(dot(dot(P.T, A), P)))

        # Gradient in F
        HF = (dot(P, nu.T) - R) > 0
        dNuF = dot(HF.T, P)
        dPF = dot(HF, nu)

        # One step of gradient descent
        P = P - a * dPE - b * dPF
        nu = nu - a * dNuE - b * dNuF

        # Clipping
        nu = clip(nu, 0, Inf)
        P = clip(P, 0, 1)

    # Now we complete
    nu = hstack((nu, (R - dot(P, nu.T)).T))
    P = hstack((P, eye(N)))
    print E, F

    return nu, P

def mixture_process(nu, P, tauc, t):
    '''
    Generate correlated spike trains from a mixture process.
    nu = rates of source spike trains
    P = mixture matrix
    tauc = correlation time constant
    t = duration
    Returns a list of (neuron_number,spike_time) to be passed to SpikeGeneratorGroup.
    '''
    n = array(poisson(nu * t)) # number of spikes for each source spike train
    if n.ndim == 0:
        n = array([n])
    # Only non-zero entries:
    nonzero = n.nonzero()[0]
    n = n[nonzero]
    P = array(P.take(nonzero, axis=1))
    nik = binomial(n, P) # number of spikes from k in i
    result = []
    for k in xrange(P.shape[1]):
        spikes = rand(n[k]) * t
        for i in xrange(P.shape[0]):
            m = nik[i, k]
            if m > 0:
                if tauc > 0:
                    selection = sample(spikes, m) + array(exponential(tauc, m))
                else:
                    selection = sample(spikes, m)
                result.extend(zip([i] * m, selection))
    result = [(i,t*second) for i,t in result]
    return result

if __name__ == '__main__':
    from time import time
    R = 2 + rand(5)
    C = rand(5, 5)
    C = .5 * (C + C.T)
    t1 = time()
    nu, P = find_mixture(R, C)
    t2 = time()
    print t2 - t1



'''
This example implements a band pass filter whose center frequency is modulated by
a sinusoid function. This modulator is implemented as a
:class:`~brian.hears.FunctionFilterbank`. One  state variable (here time) must
be kept; it is therefore implemented with a class.
The bandpass filter coefficients update is an example of how to use a
:class:`~brian.hears.ControlFilterbank`. The bandpass filter is a basic
biquadratic filter for which the Q factor and the center
frequency must be given. The input is a white noise.
'''

from brian import *
from brian.hears import *


samplerate = 20*kHz
SoundDuration = 300*ms
sound = whitenoise(SoundDuration, samplerate).ramp() 

#number of frequency channel (here it must be one as a spectrogram of the
#output is plotted)
nchannels = 1   

fc_init = 5000*Hz   #initial center frequency of the band pass filter
Q = 5               #quality factor of the band pass filter
update_interval = 1 # the filter coefficients are updated every sample

mean_center_freq = 4*kHz #mean frequency around which the CF will oscillate
amplitude = 1500*Hz      #amplitude of the oscillation
frequency = 10*Hz        #frequency of the oscillation

#this class is used in a FunctionFilterbank (via its __call__). It outputs the
#center frequency of the band pass filter. Its output is thus later passed as
#input to the controler. 
class CenterFrequencyGenerator(object):
    def __init__(self): 
        self.t=0*second
   
    def __call__(self, input):
        #update of the center frequency
        fc = mean_center_freq+amplitude*sin(2*pi*frequency*self.t)
        #update of the state variable
        self.t = self.t+1./samplerate 
        return fc

center_frequency = CenterFrequencyGenerator()      

fc_generator = FunctionFilterbank(sound, center_frequency)

#the updater of the controller generates new filter coefficient of the band pass
#filter based on the center frequency it receives from the fc_generator
#(its input)
class CoeffController(object):
    def __init__(self, target):
        self.BW = 2*arcsinh(1./2/Q)*1.44269
        self.target=target
        
    def __call__(self, input):
        fc = input[-1,:] #the control variables are taken as the last of the buffer
        w0 = 2*pi*fc/array(samplerate)    
        alpha = sin(w0)*sinh(log(2)/2*self.BW*w0/sin(w0))
        
        self.target.filt_b[:, 0, 0] = sin(w0)/2
        self.target.filt_b[:, 1, 0] = 0
        self.target.filt_b[:, 2, 0] = -sin(w0)/2
     
        self.target.filt_a[:, 0, 0] = 1+alpha
        self.target.filt_a[:, 1, 0] = -2*cos(w0)
        self.target.filt_a[:, 2, 0] = 1-alpha

# In the present example the time varying filter is a LinearFilterbank therefore
#we must initialise the filter coefficients; the one used for the first buffer computation
w0 = 2*pi*fc_init/samplerate
BW = 2*arcsinh(1./2/Q)*1.44269
alpha = sin(w0)*sinh(log(2)/2*BW*w0/sin(w0))

filt_b = zeros((nchannels, 3, 1))
filt_a = zeros((nchannels, 3, 1))

filt_b[:, 0, 0] = sin(w0)/2
filt_b[:, 1, 0] = 0
filt_b[:, 2, 0] = -sin(w0)/2

filt_a[:, 0, 0] = 1+alpha
filt_a[:, 1, 0] = -2*cos(w0)
filt_a[:, 2, 0] = 1-alpha

#the filter which will have time varying coefficients
bandpass_filter = LinearFilterbank(sound, filt_b, filt_a)
#the updater
updater = CoeffController(bandpass_filter)

#the controller. Remember it must be the last of the chain
control = ControlFilterbank(bandpass_filter, fc_generator, bandpass_filter,
                            updater, update_interval)   
      
time_varying_filter_mon = control.process()

figure(1)
pxx, freqs, bins, im = specgram(squeeze(time_varying_filter_mon),
                                NFFT=256, Fs=samplerate, noverlap=240)
imshow(flipud(pxx), aspect='auto')

show()



# coding: latin-1
"""
This is an implementation of a benchmark described
in the following review paper:

Simulation of networks of spiking neurons: A review of tools and strategies (2006).
Brette, Rudolph, Carnevale, Hines, Beeman, Bower, Diesmann, Goodman, Harris, Zirpe,
Natschläger, Pecevski, Ermentrout, Djurfeldt, Lansner, Rochel, Vibert, Alvarez, Muller,
Davison, El Boustani and Destexhe.
Journal of Computational Neuroscience

Benchmark 1: random network of integrate-and-fire neurons with exponential synaptic conductances

Clock-driven implementation with Euler integration
(no spike time interpolation)

R. Brette - Dec 2007

About 5.5s.
With compilation: 4s
"""

#import brian_no_units

from brian import *
import time
#set_global_preferences(useweave=True)

# Time constants
taum = 20 * msecond
taue = 5 * msecond
taui = 10 * msecond
# Reversal potentials
Ee = (0. + 60.) * mvolt
Ei = (-80. + 60.) * mvolt

start_time = time.time()
eqs = Equations('''
dv/dt = (-v+ge*(Ee-v)+gi*(Ei-v))*(1./taum) : volt
dge/dt = -ge*(1./taue) : 1
dgi/dt = -gi*(1./taui) : 1 
''')
# NB 1: conductances are in units of the leak conductance
# NB 2: multiplication is faster than division

P = NeuronGroup(4000, model=eqs, threshold=10 * mvolt, \
              reset=0 * mvolt, refractory=5 * msecond,
              order=1, compile=True)
#STOPPPPPP
Pe = P.subgroup(3200)
Pi = P.subgroup(800)
Ce = Connection(Pe, P, 'ge')
Ci = Connection(Pi, P, 'gi')
we = 6. / 10. # excitatory synaptic weight (voltage)
wi = 67. / 10. # inhibitory synaptic weight
Ce.connect_random(Pe, P, 0.02, weight=we)
Ci.connect_random(Pi, P, 0.02, weight=wi)
# Initialization
P.v = (randn(len(P)) * 5 - 5) * mvolt
P.ge = randn(len(P)) * 1.5 + 4
P.gi = randn(len(P)) * 12 + 20

# Record the number of spikes
Me = PopulationSpikeCounter(Pe)
Mi = PopulationSpikeCounter(Pi)

print "Network construction time:", time.time() - start_time, "seconds"
print "Simulation running..."
start_time = time.time()

run(1 * second)
duration = time.time() - start_time
print "Simulation time:", duration, "seconds"
print Me.nspikes, "excitatory spikes"
print Mi.nspikes, "inhibitory spikes"



'''
Current progress:

+ LinearStateUpdater, does everything
    - not tested with multiple state variables yet, and this might be
      wrong (might be using the transpose of the matrix)
+ Threshold, working
+ Reset, working
+ CircularVector (only for dtype=int), working
+ SpikeContainer, working
+ NeuronGroup, only some bits implemented
    - so far a NeuronGroup has state, state updater, threshold and reset,
      the latter three all optional. It also has a SpikeContainer.
    - NeuronGroup has update and reset functions which work.
+ StateMonitor, very crude
    - Can only record a given state variable for all neurons at all
      time steps
    - Can only get the values via __getitem__(i) and this returns a
      list rather than an array
    - Is derived from an essentially empty NetworkOperation class
      which doesn't include the when option at the moment
+ Network
    - Can add and run multiple NeuronGroup and NetworkOperation objs.
    - No clocks used

Still missing:

+ Clocks
+ Connection
+ (everything else in Brian)

Notes:

Ideas:

+ An interesting alternative to generating SpikeLists as in Brian is to
  just return a structure that points directly to the underlying
  CircularVector array. As long as the rest of the C++ code is aware
  that it is just getting a pointer to a CircularVector array you can
  thereby avoid a hell of a lot of copy operations.
'''
from brian import *
import brianlib as bl
import time
duration = 0.1 * second
N = 10
doplot = True
domonitor = True
debugmode = False
if debugmode:
    log_level_info()
######### Define a network we want to simulate ###############
eqs = '''
dV/dt = -(V-11*mV)/(10*ms) : volt
dW/dt = -(W-5*mV)/(50*ms) : volt
'''
G = NeuronGroup(N, eqs, threshold=10 * mV, reset=0 * mV, refractory=5 * ms)
G.V = rand(N) * 10 * mV
G.W = rand(N) * 5 * mV
if domonitor:
    M = StateMonitor(G, 'V', record=True)
    M2 = StateMonitor(G, 'W', record=True)
net = Network()
net.add(G)
if domonitor:
    net.add(M)
    net.add(M2)
######### Convert to C++ versions from brianlib ##############
# This code is lengthy, but is obviously easily automatable (a user
# would certainly never have to do anything like this).
c = array(G._state_updater._C.flatten()) # if we don't do this the memory is corrupted
if debugmode:
    print 'c.flags', c.flags
    print 'A.flags', G._state_updater.A.flags
blGsu = bl.LinearStateUpdater(G._state_updater.A, c)
blGthr = bl.Threshold(G._threshold.state, float(G._threshold.threshold))
#blGreset = bl.Reset(G._resetfun.state, float(G._resetfun.resetvalue))
period = int(G._resetfun.period / G.clock.dt) + 1
blGreset = bl.Refractoriness(G._resetfun.state, float(G._resetfun.resetvalue), period)
blG = bl.NeuronGroup(G._S, blGsu, blGthr, blGreset, G.LS.S.n, G.LS.ind.n)
if debugmode: print "brianlib.NeuronGroup instantiated OK:", G.LS.S.n, G.LS.ind.n
if domonitor:
    blM = bl.StateMonitor(blG, 0)
    blM2 = bl.StateMonitor(blG, 1)
blnet = bl.Network()
blnet.add(blG)
if domonitor:
    blnet.add(blM)
    blnet.add(blM2)
if debugmode:
    print "brianlib.Network instantiated OK"
    blG.update()
    print 'test update completed'
    blG.reset()
    print 'test reset completed'
########## Run the network in C++ and Brian ##################
# make a copy of V so we can run it twice
V = copy(G.V)
W = copy(G.W)
# Run in C++
start = time.time()
blnet.run(int(duration / defaultclock.dt))
end = time.time()
print 'N =', N, 'duration =', duration, 'domonitor =', domonitor
print 'C++ time:', (end - start) * second
if doplot and domonitor:
    subplot(221)
    for i in range(10):
        plot(blM[i])
    title('V, C++')
    subplot(222)
    for i in range(10):
        plot(blM2[i])
    title('W, C++')
# Run in Brian
G.V = V
G.W = W
start = time.time()
net.run(duration)
end = time.time()
print 'Brian time:', (end - start) * second
if doplot and domonitor:
    subplot(223)
    for i in range(10):
        plot(M[i])
    title('V, Brian')
    subplot(224)
    for i in range(10):
        plot(M2[i])
    title('W, Brian')
    show()
if debugmode and domonitor:
    pass



# -*- coding:utf-8 -*-
"""
Group delay of filters.

This gives really noisy outputs. I added a Gaussian filter.

I have a doubt: is group delay the same as narrow band ITD?
No! It's the characteristic delay!

Actually, group delay computes the characteristic delay, not the ITD.
* I think the ITD is the delay with phase consistent with IPD that
is closest to the group delay. *

[WRONG REMARKS BELOW!]
A simple observation:
* characteristic delay is just the mean ITD in a frequency band
* characteristic phase is?
ITD = -dphi/(2*pi*df) (=CD) (more precisely, group delay)
IPD = IPD(f)-ITD*2*pi*df
CP = IPD(f) - dphi/df * f (phase at 0 freq)
   = IPD(f) + CD *2*pi*f

Other idea:
* linear regression of IPD vs frequency in S1, with frequency windows in
  log or ERB space (e.g. 1/3 octave)
  Trick: nonlinear least square regression on vector (sin(phi),cos(phi)) (same as regression
  in complex plane) (could done recursively)
* could be weighted by power at that frequency (or thresholded, e.g., only
  include 50% best frequencies)
"""
from brian import *
from brian.hears.newversion import *
import os
from scipy.signal import freqz

def group_delay(f):
    '''
    Group delay of a filter.
    Group delay is -d{angle(w)}/dw.
    Smith algorithm.
    '''
    # First compute the FFT
    x=array(f).flatten()
    #x+=max(x)*(1e-20*rand(len(x))-.5*1e-20) # avoid divisions by 0
    dt=1./f.samplerate
    freq=fftfreq(len(x),dt)[1:len(x)/2]

    a=fft(x*arange(len(x)))
    b=fft(x)

    grpd=dt*(a/b).real[1:len(x)/2] # group delay

    # Smoothing
    width_dt=20
    filter='flat'
    window = {'gaussian': exp(-arange(-2 * width_dt, 2 * width_dt + 1) ** 2 * 1. / (2 * (width_dt) ** 2)),
                'flat': ones(width_dt)}[filter]
    grpd=convolve(grpd, window * 1. / sum(window), mode='same')   
    
    # Selection
    #ind=(freq>200) & (freq<3000)
    #freq=freq[ind]
    #grpd=grpd[ind]
    return freq,grpd

def ITD(fL,fR):
    '''
    ITD vs. frequency using group delays
    
    Doesn't work at all!
    '''
    dt=1/fL.samplerate
    fL=array(fL).flatten()
    fR=array(fR).flatten()
    n=len(fL)
    b=fft(fR)/fft(fL)
    a=fft(arange(n)*ifft(b))
    freq=fftfreq(n,dt)
    return freq[:n/2],dt*(a/b).real[:n/2] # only positive frequencies

def IPD_raw(fL,fR):
    dt=1/fL.samplerate
    fL=array(fL).flatten()
    fR=array(fR).flatten()
    n=len(fL)
    xR,xL=fft(fR)[1:n/2],fft(fL)[1:n/2]
    x=xR/xL
    freq=fftfreq(n,dt)[1:n/2]
    return freq,-angle(x) # is it?

def IPD(fL,fR,threshold=0.1):
    '''
    IPD vs. frequency (by unwrapping).
    Maybe: we should look at the power in these frequencies and threshold.
    There's a magnitude threshold.
    
    An ITD-based unwrapping could be chosen, i.e. minimize difference with
    ITD expectation.
    '''
    dt=1/fL.samplerate
    fL=array(fL).flatten()
    fR=array(fR).flatten()
    n=len(fL)
    xR,xL=fft(fR)[1:n/2],fft(fL)[1:n/2]
    x=xR/xL
    freq=fftfreq(n,dt)[1:n/2]
    ind=(abs(xR)>threshold) & (abs(xL)>threshold)
    unwrapped=unwrap(angle(x[ind]))
    y=zeros(n/2-1)
    y[ind]=unwrapped
    # Fill blanks (assumes linear phase + constant ITD on boundaries)
    ind=find(ind)
    breaks=find(diff(ind)>1)
    for i in range(len(breaks)):
        start=ind[breaks[i]]+1
        end=ind[breaks[i]+1] # not included
        y[start:end]=linspace(y[start-1],y[end],end-start)
    if len(ind)>0:
        if ind[0]>0:
            y[:ind[0]]=freq[:ind[0]]*y[ind[0]]/freq[ind[0]]
        if ind[-1]<len(y):
            y[ind[-1]+1:]=freq[ind[-1]+1:]*y[ind[-1]]/freq[ind[-1]]
    #subplot(211)
    #plot(freq,abs(fft(fL)[:n/2]))
    #plot(freq,abs(fft(fR)[:n/2]))
    
    return freq,y

def ITD2(fL,fR):
    '''
    ITD vs. frequency
    '''
    dt=1/fL.samplerate
    fL=array(fL).flatten()
    fR=array(fR).flatten()
    n=len(fL)
    x=fft(fR)/fft(fL)
    freq=fftfreq(len(x),dt)[:n/2]
    ipd=unwrap(angle(x)[:n/2])
    itd=-diff(ipd)/(2*pi*diff(freq))
    return freq[:-1],itd

def ITD3(fL,fR,threshold=0.1):
    freq,ipd=IPD(fL,fR,threshold)
    # Smoothing (ITD rather than IPD is smoothed, i.e., assuming small variations in ITD)
    # Width should be in ERB space
    # Differentiation could be done without the blank filling
    width_dt=10
    filter='flat'
    window = {'gaussian': exp(-arange(-2 * width_dt, 2 * width_dt + 1) ** 2 * 1. / (2 * (width_dt) ** 2)),
                'flat': ones(width_dt)}[filter]
    #ipd=convolve(ipd/(2*pi*freq), window * 1. / sum(window), mode='same')*freq
    ipd=convolve(ipd/(2*pi), window * 1. / sum(window), mode='same')
    # I suppose we need a window?

    #itd=ipd/(2*pi*freq) # actually is that the ITD?? maybe not!
    itd=zeros(len(ipd))
    itd[:-1]=-diff(ipd)/diff(freq)
    itd[-1]=itd[-2]
    y = itd

    return freq,y

def ITD4(fL,fR):
    '''
    ITD closest to group delay consistent with IPD
    '''
    freq,gR=group_delay(fR)
    _,gL=group_delay(fL)
    grpd=gR-gL
    grpd_phase=(((grpd*freq+.5) % 1) -.5)*2*pi
    _,ipd=IPD_raw(fL,fR) # could filter this too (-pi..pi)
    ipd[(ipd-grpd_phase)>pi]-=2*pi
    ipd[(ipd-grpd_phase)<-pi]+=2*pi
    itd=grpd+(ipd-grpd_phase)/(2*pi*freq)
    return itd

choice='IRCAM'
hrtf_locations = [
        r'C:\HRTF\\' + choice,
        r'D:\HRTF\\' + choice,
        r'/home/bertrand/Data/Measurements/HRTF/' + choice,
        r'C:\Documents and Settings\dan\My Documents\Programming\\' + choice
        ]
found = 0
for path in hrtf_locations:
    if os.path.exists(path):
        found = 1
        break
if found == 0:
    raise IOError('Cannot find IRCAM HRTF location, add to ircam_locations')

ircam = IRCAM_LISTEN(path)
h = ircam.load_subject(1002)
h = h.subset(lambda azim,elev:azim==120 and elev==30)

#freq,xL=group_delay(h.hrtf[0].left)
#_,xR=group_delay(h.hrtf[0].right)
#itd=(xR-xL)
left=h.hrtf[0].left.flatten()
right=h.hrtf[0].right.flatten()
freq,itd=ITD3(left,right,threshold=0.05)
itd2=group_delay(right)[1]-group_delay(left)[1]
itd4=ITD4(left,right)
_,ipd=IPD_raw(left,right)#,threshold=0.0)
#ind=(freq>200) & (freq<3000)
#freq=freq[ind]
#itd=itd[ind]
#subplot(211)
#plot(freq,itd*1e6)
subplot(312)
ind=(freq<5000) & (freq>180)
#plot(freq[ind],itd[ind]*1e6,'r') # CD
plot(freq[ind],itd2[ind]*1e6,'r')
#plot(freq[ind],itd4[ind]*1e6,'r')
plot(freq[ind],unwrap(ipd[ind])/(2*pi*freq[ind])*1e6,'b') # ITD
#plot(freq[ind],(group_delay(right)[1][ind]-group_delay(left)[1][ind])*1e6,'k')
#ylim(0,1500)
subplot(311)
plot(freq[ind],ipd[ind]) # IPD
subplot(313)
CP = IPD_raw(left,right)[1] + itd *2*pi*freq # sure about this??
hist((((CP[ind]+pi) % (2*pi))-pi)/(2*pi))
#plot(freq[ind],(((CP[ind]+pi) % (2*pi))-pi)/(2*pi)) # CP
#ylim(-0.5,0.5)
#plot(smooth(smooth(smooth(smooth(freq)))),smooth(smooth(smooth(smooth(ITD)))))

#plot(h.hrtf[0].left)
#plot(h.hrtf[0].right)
show()



#return self.X[range(i0,n)+range(0,j0)] # Costly operation!
##return concatenate((self.X[i0:],self.X[0:j0])) # might be slightly faster
##return self.X.take(range(i0,j0+n),mode='wrap') # does not seem to work

from pylab import *
import time, timeit
from numpy import zeros, hstack, concatenate

def f_hstack(n, m, repeats=100000):
    x = zeros(n)
    start = time.time()
    for _ in xrange(repeats):
        hstack((x[m:], x[:m]))
    end = time.time()
    return (end - start) / repeats

def f_concatenate(n, m, repeats=100000):
    x = zeros(n)
    start = time.time()
    for _ in xrange(repeats):
        concatenate((x[m:], x[:m]))
    end = time.time()
    return (end - start) / repeats

def f_range(n, m, repeats=100000):
    x = zeros(n)
    start = time.time()
    for _ in xrange(repeats):
        x[range(m, n) + range(m)]
    end = time.time()
    return (end - start) / repeats

def f_take(n, m, repeats=100000):
    x = zeros(n)
    start = time.time()
    for _ in xrange(repeats):
        x.take(arange(m, m + n), mode='wrap')
    end = time.time()
    return (end - start) / repeats

N = [1, 2, 3, 4, 5, 10, 100]
P = [.1, .25, .5, .75]

for p in P:
    print p
    t_hstack = []
    t_concatenate = []
    t_range = []
    t_take = []
    for n in N:
        print n
        m = int(n * p)
        t_hstack.append(f_hstack(n, m))
        t_concatenate.append(f_concatenate(n, m))
        t_range.append(f_range(n, m))
        t_take.append(f_take(n, m))
    plot(N, t_hstack, '-r', label=str(p))
    plot(N, t_concatenate, '-g', label=str(p))
    plot(N, t_range, '-b', label=str(p))
    plot(N, t_take, '-k', label=str(p))
#legend()
show()



'''
Tutorial 2a: The concept of a Connection
****************************************

The network
~~~~~~~~~~~

In this first part, we'll build a network consisting of three neurons. The
first two neurons will be under direct control and have no equations
defining them, they'll just produce spikes which will feed into the third
neuron. This third neuron has two different state variables, called Va
and Vb. The first two neurons will be connected to the third neuron, but
a spike arriving at the third neuron will be treated differently
according to whether it came from the first or second neuron (which you
can consider as meaning that the first two neurons have different
types of synapses on to the third neuron).

The program starts as follows.
'''
from brian import *

tau_a = 1 * ms
tau_b = 10 * ms
Vt = 10 * mV
Vr = 0 * mV
'''
Differential equations
~~~~~~~~~~~~~~~~~~~~~~

This time, we will have multiple differential equations. We will use the
:class:`Equations` object, although you could equally pass the multi-line string
defining the differential equations directly when initialising the :class:`NeuronGroup`
object (see the next part of the tutorial for an example of this).
'''
eqs = Equations('''
      dVa/dt = -Va/tau_a : volt
      dVb/dt = -Vb/tau_b : volt
      ''')
'''
So far, we have defined a model neuron with two state variables, ``Va``
and ``Vb``, which both decay exponentially towards 0, but with different
time constants ``tau_a`` and ``tau_b``. This is just so that you can see
the difference between them more clearly in the plot later on.

SpikeGeneratorGroup
~~~~~~~~~~~~~~~~~~~

Now we introduce the :class:`SpikeGeneratorGroup` class. This is a group of
neurons without a model, which just produces spikes at the times
that you specify. You create a group like this by writing::

    G = SpikeGeneratorGroup(N,spiketimes)

where ``N`` is the number of neurons in the group, and ``spiketimes`` is a
list of pairs ``(i,t)`` indicating that neuron ``i`` should fire at time ``t``.
In fact, ``spiketimes`` can be any 'iterable container' or 'generator',
but we don't cover that here (see the detailed documentation for
:class:`SpikeGeneratorGroup`).

In our case, we want to create a group with two neurons, the first
of which (neuron 0) fires at times 1 ms and 4 ms, and the second
of which (neuron 1) fires at times 2 ms and 3 ms. The list of
``spiketimes`` then is:
'''
spiketimes = [(0, 1 * ms), (0, 4 * ms),
              (1, 2 * ms), (1, 3 * ms)]
'''
and we create the group as follows:
'''
G1 = SpikeGeneratorGroup(2, spiketimes)
'''
Now we create a second group, with one neuron, according to the
model we defined earlier.
'''
G2 = NeuronGroup(N=1, model=eqs, threshold=Vt, reset=Vr)
'''
Connections
~~~~~~~~~~~

In Brian, a :class:`Connection` from one :class:`NeuronGroup` to another is
defined by writing::

    C = Connection(G,H,state)

Here ``G`` is the source group, ``H`` is the target group, and ``state`` is the
name of the target state variable. When a neuron ``i`` in ``G`` fires, Brian
finds all the neurons ``j`` in ``H`` that ``i`` in ``G`` is connected to, and adds
the amount ``C[i,j]`` to the specified state variable of neuron ``j`` in ``H``.
Here ``C[i,j]`` is the (i,j)th entry of the connection matrix of ``C`` (which
is initially all zero).

To start with, we create two connections from the group of two
directly controlled neurons to the group of one neuron with the
differential equations. The first connection has the target state ``Va``
and the second has the target state ``Vb``.
'''
C1 = Connection(G1, G2, 'Va')
C2 = Connection(G1, G2, 'Vb')
'''
So far, this only declares our intention to connect neurons in group
``G1`` to neurons in group ``G2``, because the connection matrix is initially
all zeros. Now, with connection ``C1`` we connect neuron 0 in group ``G1``
to neuron 0 in group ``G2``, with weight 3 mV. This means that when neuron
0 in group ``G1`` fires, the state variable ``Va`` of the neuron in group ``G2``
will be increased by 6 mV. Then we use connection ``C2`` to connection
neuron 1 in group ``G1`` to neuron 0 in group ``G2``, this time with weight
3 mV.
'''
C1[0, 0] = 6 * mV
C2[1, 0] = 3 * mV
'''
The net effect of this is that when neuron 0 of ``G1`` fires, ``Va`` for
the neuron in ``G2`` will increase 6 mV, and when neuron 1 of ``G1`` fires,
``Vb`` for the neuron in ``G2`` will increase 3 mV.

Now we set up monitors to record the activity of the network,
run it and plot it.
'''
Ma = StateMonitor(G2, 'Va', record=True)
Mb = StateMonitor(G2, 'Vb', record=True)

run(10 * ms)

plot(Ma.times, Ma[0])
plot(Mb.times, Mb[0])
show()
'''
.. image:: images/tutorials/2a.jpg

The two plots show the state variables ``Va`` and ``Vb`` for the single
neuron in group ``G2``. ``Va`` is shown in blue, and ``Vb`` in green.
According to the differential equations, ``Va`` decays much faster
than ``Vb`` (time constant 1 ms rather than 10 ms), but we have set
it up (through the connection strengths) that an incoming
spike from neuron 0 of ``G1`` causes a large increase of 6 mV to ``Va``,
whereas a spike from neuron 1 of ``G1`` causes a smaller increase of
3 mV to Vb. The value for ``Va`` then jumps at times 1 ms and 4 ms,
when we defined neuron 0 of ``G1`` to fire, and decays almost back
to rest in between. The value for ``Vb`` jumps at times 2 ms and
3 ms, and because the times are closer together and the time
constant is longer, they add together.

In the next part of this tutorial, we'll see how to use this
system to do something useful.

Exercises
~~~~~~~~~

1. Try playing with the parameters ``tau_a``, ``tau_b`` and the connection
   strengths, ``C1[0,0]`` and ``C2[0,1]``. Try changing the list of
   spike times.
2. In this part of the tutorial, the states ``Va`` and ``Vb`` are
   independent of one another. Try rewriting the differential
   equations so that they're not independent and play around
   with that.
3. Write a network with inhibitory and excitatory neurons. Hint: you
   only need one connection.
4. Write a network with inhibitory and excitatory neurons whose
   actions have different time constants (for example, excitatory
   neurons have a slower effect than inhibitory ones).
   
Solutions
~~~~~~~~~

3. Simple write ``C[i,j]=-3*mV`` to make the connection from neuron
   i to neuron j inhibitory.
4. See the next part of this tutorial.
'''



'''
Automatic differentiation.
(C) R. Brette 2008
This file is released under the terms of the BSD licence
--------------------------------------------------------
Method: forward accumulation by operator overloading.

Use::
  df=differentiate(f)                  # where f is a single-variable function
  y=differentiate(f,x0)                # differentiate at x=x0
  y=differentiate(f,x0,order=2)        # second-order derivative at x=x0
  y=differentiate(f,(a,b),order=(1,2)) # d2f/dxdy(a,b)
  y=differentiate(f,order=(1,2))       # d2f/dxdy
  
  a0,a1,a2=taylor_series(f,x0,order=2) # 2nd order Taylor series of f at x0
  J=gradient(f,(a,b))                  # gradient of f at (a,b) (returns a list)
  H=hessian(f,(a,b))                   # hessian of f at (a,b) (returns a numpy array)
  
TODO:
* replace x by x0
* merge HigherDifferentiable in Differentiable?
* simplify code
* gradient, taylor_series, hessian: x0 should be optional

* iadd, etc. (more operators)
* NonDifferentiableException and maybe use this on conditions (when cmp=0)
* Differential operators: Laplacian...
* gradient, taylor_series, hessian: x0 should be optional
* test with units, functions Rp->something, alternative differential operators,
  differential (f:Rp->Rn), complex numbers, arithmetic derivative
* remove most dependency with numpy (except Hessian)
* implicitly defined functions

If several partial derivatives of the same function need to be
calculated, it is faster to use the lower level object Differentiable.
Example::
  y=f(Differentiable('x',3),Differentiable('y',2))
will return a Differentiable object with y.val=f(3,2)
and y.diff={'x': df/dx(3,2), 'y': df/dy(3,2)}
'''

from numpy import exp, sin, cos, log, zeros # todo: look in frame instead?
import types

__all__ = ['differentiate', 'Differentiable', 'taylor_series', 'gradient', 'hessian']

def differentiate(f, x=None, order=1):
    '''
    Calculate the derivative of f at point x.
    Higher-order derivatives are possible.
    Ex.: differentiate(lambda x:3*x*x+2,x=2,order=2)
    (Returns: 6.0)
    
    Partial derivatives:
    1) Give a tuple with the order.
    Ex.: differentiate(lambda x,y:x*y+2*y+1,x=(1,2),order=(0,1))
    (Returns: 3.0 (partial derivative d/dy) )
    2) Give a dictionnary with the order:
    Ex.: differentiate(lambda x,y:x*y+2*y+1,x=(1,2),order={'y':1})
    '''
    if x is None:
        return lambda x:differentiate(f, x, order)
    if (type(order) == types.ListType) or (type(order) == types.TupleType): # several variables
        # Build the list of arguments
        n = sum(order) # total order
        args = [HigherDifferentiable(i, x[i], n) for i in range(len(x))]
        y = f(*args)
        for i in xrange(len(x)):
            for _ in xrange(order[i]):
                if isinstance(y, Differentiable) and (i in y.diff):
                    y = y.diff[i]
                else: # constant
                    return 0.
        return y
    elif (type(order) == types.DictType): # named variables
        # Build the list of arguments
        n = sum(order.itervalues())
        vars = list(f.func_code.co_varnames)
        args = []
        args.extend(x)
        for name in order.iterkeys():
            i = vars.index(name)
            args[i] = HigherDifferentiable(name, x[i], n)
        y = f(*args)
        for name, n in order.iteritems():
            for j in range(n):
                if isinstance(y, Differentiable) and (name in y.diff):
                    y = y.diff[name]
                else: # constant
                    return 0.
        return y
    elif order == 0:
        return f(x)
    elif order == 1:
        y = f(Differentiable('x', x))
        if isinstance(y, Differentiable) and ('x' in y.diff):
            return y.diff['x']
        else: # constant
            return 0.
    else:
        return differentiate(lambda y:differentiate(f, y, order=order - 1), x)

def taylor_series(f, x, order=1):
    '''
    Returns the list of coefficients of the Taylor Series of f
    at x (scalar).
    '''
    y = f(HigherDifferentiable('x', x, order))
    series = []
    n = 1.
    for i in range(order + 1):
        # Value
        val = y
        while isinstance(val, Differentiable):
            val = val.val
        series.append(val / n)
        n *= (i + 1)
        # Next order
        if isinstance(y, Differentiable) and ('x' in y.diff):
            y = y.diff['x']
        else:
            y = 0.
    return series

def gradient(f, x):
    '''
    Gradient of f at x (= tuple).
    Result = list.
    '''
    args = [Differentiable(i, val=x[i]) for i in range(len(x))]
    y = f(*args)
    if isinstance(y, Differentiable):
        result = []
        for i in range(len(x)):
            if i in y.diff:
                result.append(y.diff[i])
            else:
                result.append(0.)
        return result
    else: # constant
        return [0.] * len(x)

def hessian(f, x):
    '''
    Hessian of f at x (= tuple).
    Result = array.
    N.B.: not working with vectors.
    '''
    args = [HigherDifferentiable(i, x[i], 2) for i in range(len(x))]
    y = f(*args)
    result = zeros((len(x), len(x)))
    if isinstance(y, Differentiable): # non-zero Hessian
        for i in range(len(x)):
            if i in y.diff and isinstance(y.diff[i], Differentiable):
                for j in range(len(x)):
                    if j in y.diff[i].diff:
                        result[i, j] = y.diff[i].diff[j]
    return result


class Differentiable(object):
    '''
    A differentiable variable.
    Implemented operations: +,-,*,[],len,exp,sin,cos,/,abs,sqrt,comparisons,**
    '''
    def __init__(self, name=None, val=0.):
        '''
        Initializes a variable and its derivative.
        If the name is None, then it is a constant.
        '''
        self.val = val # value
        if name == None: # constant
            self.diff = {} # derivative
        else:
            self.diff = {name:1.} # or jacobian? (eye)

    def sqrt(self):
        return self ** .5

    def __cmp__(self, x):
        if isinstance(x, Differentiable):
            return cmp(self.val, x.val)
        else:
            return cmp(self.val, x)

    def __abs__(self):
        # NOT WORKING WITH VECTORS
        if self.val == 0:
            # TODO: specific exception
            raise ZeroDivisionError, "x:abs(x) is not differentiable at x=0."
        elif self.val > 0:
            return self
        else:
            return - self

    def __add__(self, y):
        # VECTOR-READY
        if isinstance(y, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(y.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in y.diff):
                    zdict[key] = self.diff[key] + y.diff[key]
            z = Differentiable()
            z.val = self.val + y.val
            z.diff = zdict
            return z
        else: # Adding a constant
            z = Differentiable(val=self.val + y)
            z.diff = self.diff
            return z

    def __radd__(self, x):
        # VECTOR-READY
        if isinstance(x, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(x.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in x.diff):
                    zdict[key] = x.diff[key] + self.diff[key]
            z = Differentiable()
            z.val = x.val + self.val
            z.diff = zdict
            return z
        else: # Adding a constant
            z = Differentiable(val=x + self.val)
            z.diff = self.diff
            return z

    def __mul__(self, y):
        # !Not working with vectors!
        if isinstance(y, Differentiable):
            #TODO: with vectors
            zdict = {}
            zdict.update(self.diff)
            zdict.update(y.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in y.diff):
                    zdict[key] = self.diff[key] * y.val + self.val * y.diff[key]
                elif (key in self.diff):
                    zdict[key] = self.diff[key] * y.val
                else:
                    zdict[key] = self.val * y.diff[key]
            z = Differentiable()
            z.val = self.val * y.val
            z.diff = zdict
            return z
        else: # Multiplying by a constant
            z = Differentiable(val=self.val * y)
            for key in self.diff:
                z.diff[key] = self.diff[key] * y
            return z

    def __rmul__(self, x):
        if isinstance(x, Differentiable):
            #TODO: with vectors
            zdict = {}
            zdict.update(self.diff)
            zdict.update(x.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in x.diff):
                    zdict[key] = x.val * self.diff[key] + x.diff[key] * self.val
                elif (key in self.diff):
                    zdict[key] = x.val * self.diff[key]
                else:
                    zdict[key] = x.diff[key] * self.val
            z = Differentiable()
            z.val = x.val * self.val
            z.diff = zdict
            return z
        else: # Multiplying by a constant
            z = Differentiable(val=x * self.val)
            for key in self.diff:
                z.diff[key] = x * self.diff[key]
            return z

    def __div__(self, x):
        return self * (x ** -1)

    def __rdiv__(self, x):
        return x * (self ** -1)

    def __sub__(self, y):
        # VECTOR-READY
        if isinstance(y, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(y.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in y.diff):
                    zdict[key] = self.diff[key] - y.diff[key]
            z = Differentiable()
            z.val = self.val - y.val
            z.diff = zdict
            return z
        else: # Subtracting a constant
            z = Differentiable(val=self.val - y)
            z.diff = self.diff
            return z

    def __rsub__(self, x):
        # VECTOR-READY
        if isinstance(x, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(x.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in x.diff):
                    zdict[key] = x.diff[key] - self.diff[key]
            z = Differentiable()
            z.val = x.val - self.val
            z.diff = zdict
            return z
        else: # Subtracting a constant
            z = Differentiable(val=x - self.val)
            for key in self.diff:
                z.diff[key] = -self.diff[key]
            return z

    def __neg__(self):
        # VECTOR-READY
        z = Differentiable(val= -self.val)
        for key in self.diff:
            z.diff[key] = -self.diff[key]
        return z

    def __pow__(self, x):
        # NOT WORKING WITH VECTORS
        if isinstance(x, Differentiable):
            return exp(x * log(self))
        elif x == 0:
            z = Differentiable(val=self.val ** x)
            for key in self.diff:
                z.diff[key] = 0 * self.diff[key]
            return z
        else:
            z = Differentiable(val=self.val ** x)
            for key in self.diff:
                z.diff[key] = x * self.val ** (x - 1) * self.diff[key]
            return z

    def __rpow__(self, x):
        # NOT WORKING WITH VECTORS
        return exp(self * log(x))

    def __getitem__(self, i):
        z = Differentiable(val=self.val[i])
        z.diff = {}
        for key in self.diff.iterkeys():
            z.diff[key] = self.diff[key][i, :]
        return z

    def __len__(self):
        return len(self.val)

    def __str__(self):
        s = str(self.val)
        for key, value in self.diff.iteritems():
            s += ' ; d/d' + key + '=' + str(value)
        return s

    def exp(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = exp(self.val) * self.diff[key]
        z = Differentiable(val=exp(self.val))
        z.diff = zdict
        return z

    def cos(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = -sin(self.val) * self.diff[key]
        z = Differentiable(val=cos(self.val))
        z.diff = zdict
        return z

    def sin(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = cos(self.val) * self.diff[key]
        z = Differentiable(val=sin(self.val))
        z.diff = zdict
        return z

    def log(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = (1. / (self.val)) * self.diff[key]
        z = Differentiable(val=log(self.val))
        z.diff = zdict
        return z


class HigherDifferentiable(Differentiable):
    '''
    A differentiable variable with order n.
    '''
    def __init__(self, name=None, val=0., order=1):
        Differentiable.__init__(self, name, val)
        if order > 1:
            self.val = HigherDifferentiable(name, val, order - 1)

if __name__ == '__main__':
    print "Derivative:"
    print "> differentiate(lambda x:3*x*x+2,x=2,order=2)"
    print differentiate(lambda x:3 * x * x + 2, x=2, order=2)
    print "> differentiate(lambda x:3*x*x+2,order=2)(2)"
    print differentiate(lambda x:3 * x * x + 2, order=2)(2)
    print "Partial derivative:"
    print "> differentiate(lambda x,y:x*y+2*y+1,x=(1,2),order=(0,1))"
    print differentiate(lambda x, y:x * y + 2 * y + 1, x=(1, 2), order=(0, 1))



from brian import *

N = 200

defaultclock.dt = 0.5 * ms

izh_eqs = '''
dv/dt=0.04/ms/mV*v**2+5/ms*v+140*mV/ms-u+ge-gi+i:volt
du/dt=a*(b*v-u):volt/second
a:1/second
b:1/second
dge/dt=-ge/ms:volt/second
dgi/dt=-gi/ms:volt/second
i:volt/second
c:volt
d:volt
'''


reset_eqs = '''
v=c;u=u+d
'''
group = NeuronGroup(200, model=izh_eqs, threshold=30 * volt, reset=reset_eqs)

sub1 = group[:100]
sub2 = group[100:]

for i in range(len(sub1)):
  sub1[i].a = 0.02 / ms
  sub1[i].b = 0.2 / ms
  sub1[i].c = -65.0 * mV
  sub1[i].d = 8.0 * mV

for i in range(len(sub2)):
  sub2[i].a = 0.02 / ms
  sub2[i].b = 0.2 / ms
  sub2[i].c = -65.0 * mV
  sub2[i].d = 8.0 * mV


#direct links
linksexcit = Connection(sub1, sub2, 'ge', structure='dense')

#CASE 1
linksexcit.connect_full(sub1, sub2, weight=lambda i, j:rand()*10.0 * mV / ms)
#linksexcit.connect_full(sub1,sub2,weight=lambda i,j:rand(array(j).size)*10.0*mV/ms)

#CASE 2
#create custom links
#for i in range(len(sub1)):
#  for j in range(len(sub2)):
#    linksexcit[i,j]=rand()*10*mV/ms

clock_i = Clock(dt=1 * ms)
@network_operation
def myoperation(clock_i):
    sub1.i = [10 * randn()*mV / ms for i in range(len(sub1))]

group.v = -65 * mV
group.u = group.v * group.b


spkm = SpikeMonitor(group)
spkc1 = SpikeCounter(sub1)
spkc2 = SpikeCounter(sub1)

run(200 * ms)
print spkc1.nspikes
print spkc2.nspikes
raster_plot()
figure()
imshow(linksexcit.W.todense())
colorbar()
show()



'''
Example of the use of the cochlear models (:class:`~brian.hears.DRNL`,
:class:`~brian.hears.DCGC` and :class:`~brian.hears.TanCarney`) available in the library.
'''
from brian import *
from brian.hears import *

simulation_duration = 50*ms
set_default_samplerate(50*kHz)
sound = whitenoise(simulation_duration)
sound = sound.atlevel(50*dB) # level in rms dB SPL
cf = erbspace(100*Hz, 1000*Hz, 50) # centre frequencies
interval = 16  #update interval of the time varying filters
## DNRL
#param_drnl = {}
#param_drnl['lp_nl_cutoff_m'] = 1.1
#drnl_filter=DRNL(sound, cf, type='human', param=param_drnl)
#out = drnl_filter.process()

## DCGC
#param_dcgc = {}
#param_dcgc['c1'] = -2.96

#dcgc_filter = DCGC(sound, cf, interval, param=param_dcgc)
#out = dcgc_filter.process()

## Tan and Carney 2003
tan_filter = TanCarney(sound, cf, interval)
out = tan_filter.process()

figure()
imshow(flipud(out.T), aspect='auto')
show()



from brian import *

N = 100

H = SpikeGeneratorGroup(1, [(0, 0 * ms)])
G = NeuronGroup(N, 'V:1', threshold=1, reset=0)

C = Connection(H, G, structure='dense', delay=True, weight=2.0, max_delay=10 * ms)
C.delay[0, :] = linspace(0 * ms, 10 * ms, N)

sp = SpikeMonitor(G)
Mv = StateMonitor(G, 'V', record=True)

#run(5*ms)
#
#imshow(C._delayedreaction, origin='lower left', interpolation='nearest')
#axhline(C._cur_delay_ind)
#show()
#exit()

run(15 * ms)

subplot(211)
raster_plot(sp)
subplot(212)
for i in range(N):
    plot(Mv.times, Mv[i] + i)
show()



'''
In this script I test the basic Synapses behavior:
synapses = syn.Synapses(gin, gout, model = 'w : 1', pre = 'v += w')
'''

from brian import *
import dev.ideas.synapses.synapses as syn
reload(syn)
log_level_debug()

reinit_default_clock()

if True:
    spikes = [(0, 10*ms), (1, 10*ms), (1, 20*ms), (2, 30*ms)]
    gin = SpikeGeneratorGroup(3, spikes)
    gout = NeuronGroup(2, model = 'dv/dt = -v/(5*ms) : 1')

    eqs = '''
w : 1
u : 1
z = u + v : 1
dx/dt = -x/(10*ms) : 1
'''
    
    pre = '''
v += w ; x += w
'''

    # init synapses
    synapses = syn.Synapses(gin, gout, model = eqs, pre = pre)
    
    # set synapses and weights
    synapses[:, 0] = True
    synapses[1, 1] = True

    synapses.w[:, 0] = 1
    synapses.w[1, 1] = 3
    
    # monitor and run
    Mv = StateMonitor(gout, 
'v', record = True)
    net = Network(gin, gout, synapses, Mv)
    net.run(100*ms)
    
    for i in range(len(gout)):
        plot(Mv.times/ms, Mv[i])
    
    show()




from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
                    reshape, sum, log, Monitor, NetworkOperation, defaultclock, linspace, vstack, \
                    arange, sort_spikes, rint, SpikeMonitor, Connection,SpikeGeneratorGroup,DelayConnection,\
                    SpikeCounter,forget,diagflat,inf,sqrt
from scipy.sparse import dia_matrix,lil_matrix
from brian.tools.statistics import firing_rate, get_gamma_factor

try:
    from playdoh import *
except Exception, e:
    print e
    raise ImportError("Playdoh must be installed (https://code.google.com/p/playdoh/)")

try:
    import pycuda
    from gpu_modelfitting import GPUModelFitting
    can_use_gpu = True
except ImportError:
    can_use_gpu = False
from brian.experimental.codegen.integration_schemes import *
import sys, cPickle

__all__ = ['modelfitting', 'print_table', 'get_spikes', 'predict', 'PSO', 'GA','CMAES','Brette','VanRossum',
           'MAXCPU', 'MAXGPU',
           'GammaFactor', 'LpError',
           'debug_level', 'info_level', 'warning_level', 'open_server']



class DataTransformer(object):
    """
    Transform spike, input and trace data from user-friendly data structures,
    like 2 dimensional arrays or lists of spikes, into algorithm- and GPU-friendly structures,
    i.e. inline vectors (1 dimensional) easily parallelizable.
    """
    def __init__(self, neurons, input, spikes = None, traces = None, dt = defaultclock.dt,
                 slices = 1, overlap = 0*ms, groups = 1):
        self.neurons = neurons # number of particles on the node
        self.input = input # a IxT array
        self.spikes = spikes # a list of spikes [(i,t)...]
        self.traces = traces # a KxT array
        self.slices = slices
        self.overlap = overlap
        self.groups = groups
        self.dt = dt
        # ensure 2 dimensions
        if self.input.ndim == 1:
            self.input = self.input.reshape((1,-1))
        if self.traces is not None:
            if self.traces.ndim == 1:
                self.traces = self.traces.reshape((1,-1))
        self.inputs_count = self.input.shape[0]
        self.T = self.input.shape[1] # number of steps
        self.duration = self.T*self.dt
        self.subpopsize = self.neurons/self.groups # number of neurons per group: nodesize/groups
        self.input = self.input[:,0:self.slices * (self.T / self.slices)] # makes sure that len(input) is a multiple of slices
        self.sliced_steps = self.T / self.slices # timesteps per slice
        self.overlap_steps = int(self.overlap / self.dt) # timesteps during the overlap
        self.total_steps = self.sliced_steps + self.overlap_steps # total number of timesteps
        self.sliced_duration = self.overlap + self.duration / self.slices # duration of the vectorized simulation
        self.N = self.neurons * self.slices # TOTAL number of neurons on this node
        self.input = hstack((zeros((self.inputs_count, self.overlap_steps)), self.input)) # add zeros at the beginning because there is no overlap from the previous slice
        
    def slice_spikes(self, spikes):
        # from standard structure to standard structure
        sliced_spikes = []
        slice_length = self.sliced_steps*self.dt
        for (i,t) in spikes:
            slice = int(t/slice_length)
            newt = self.overlap + (t % slice_length)*second
            newi = i + self.groups*slice
            sliced_spikes.append((newi, newt)) 
        sliced_spikes = sort_spikes(sliced_spikes)
        return sliced_spikes

    def slice_traces(self, traces):
        # from standard structure to standard structure
        k = traces.shape[0]
        sliced_traces = zeros((k*self.slices, self.total_steps))
        for slice in xrange(self.slices):
            i0 = slice*k
            i1 = (slice+1)*k
            j0 = slice*self.sliced_steps
            j1 = (slice+1)*self.sliced_steps
            sliced_traces[i0:i1,self.overlap_steps:] = traces[:,j0:j1]
            if slice>0:
                sliced_traces[i0:i1,:self.overlap_steps] = traces[:,j0-self.overlap_steps:j0]
        return sliced_traces

    def transform_spikes(self, spikes):
        # from standard structure to inline structure
        i, t = zip(*spikes)
        i = array(i)
        t = array(t)
        alls = []
        n = 0
        pointers = []
        model_target = []
        for j in xrange(self.groups):
            s = sort(t[i == j])
            s = hstack((-1 * second, s, self.duration + 1 * second))
            model_target.extend([j] * self.subpopsize)
            alls.append(s)
            pointers.append(n)
            n += len(s)
        pointers = array(pointers, dtype=int)
        model_target = array(hstack(model_target), dtype=int)
        spikes_inline = hstack(alls)
        spikes_offset = pointers[model_target]
        return spikes_inline, spikes_offset
    
    def transform_traces(self, traces):
        # from standard structure to inline structure
        K, T = traces.shape
        traces_inline = traces.flatten()
        traces_offset = array(kron(arange(K), T*ones(self.subpopsize)), dtype=int)
        return traces_inline, traces_offset




class Criterion(Monitor, NetworkOperation):
    """
    Abstract class from which modelfitting criterions should derive.
    Derived classes should implement the following methods:
    
    ``initialize(self, **params)``
        Called once before the simulation. ```params`` is a dictionary with criterion-specific
        parameters. 
    
    ``timestep_call(self)``
        Called at every timestep.
    
    ``spike_call(self, neurons)``
        Called at every spike, with the spiking neurons as argument.
    
    ``get_values(self)``
        Called at the end, returns the criterion values.
        
    You have access to the following methods:
    
    ``self.get_value(self, varname)``
        Returns the value of the specified variable for all neurons (vector).
    
    You have access to the following attributes:
    
    ``self.step``
        The time step (integer)
    
    ``self.group``
        The NeuronGroup
    
    ``self.traces=None``
        Target traces.
        A 2-dimensional K*T array where K is the number of targets, and T the total number of timesteps.
        It is still a 2-dimensional array when K=1. It is None if not specified.
        
    ``self.spikes=None``
        A list of target spike trains : [(i,t)..] where i is the target index and t the spike time.
    
    ``self.N``
        The number of neurons in the NeuronGroup
    
    ``self.K=1``
        The number of targets.
    
    ``self.duration``
        The total duration of the simulation, in seconds.
    
    ``self.total_steps``
        The total number of time steps.
    
    ``self.dt``
        The timestep duration, in seconds.
    
    ``self.delays=zeros(self.n)``
        The delays for every neuron, in seconds. The delay is relative to the target.
    
    ``self.onset=0*ms``
        The onset, in seconds. The first timesteps, before onset, should be discarded in the criterion.
    
    ``self.intdelays=0``
        The delays, but in number of timesteps (``int(delays/dt)``).
    
    NOTE: traces and spikes have all been sliced before being passed to the criterion object
    """
    def __init__(self, group, traces=None, spikes=None, targets_count=1, duration=None, onset=0*ms, 
                 spikes_inline=None, spikes_offset=None,
                 traces_inline=None, traces_offset=None,
                 delays=None, when='end', **params):
        NetworkOperation.__init__(self, None, clock=group.clock, when=when)
        self.group = group
        
        # needed by SpikeMonitor
        self.source = group
        self.source.set_max_delay(0)
        self.delay = 0
        
        self.N = len(group) # number of neurons
        self.K = targets_count # number of targets
        self.dt = self.clock.dt
        if traces is not None:
            self.traces = array(traces) # KxT array
            if self.traces.ndim == 1:
                self.traces = self.traces.reshape((1,-1))
            assert targets_count==self.traces.shape[0]
        self.spikes = spikes
        # get the duration from the traces if duration is not specified in the constructor
        if duration is None: duration = self.traces.shape[1] # total number of steps
        self.duration = duration
        self.total_steps = int(duration/self.dt)
        if delays is None: delays = zeros(self.n)
        self.delays = delays
        self.onset = int(onset/self.dt)
        self.intdelays = array(self.delays/self.clock.dt, dtype=int)
        self.mindelay = min(delays)
        self.maxdelay = max(delays)
        # the following data is sliced
        self.spikes_inline = spikes_inline
        self.spikes_offset = spikes_offset
        self.traces_inline = traces_inline
        self.traces_offset = traces_offset
        if self.spikes is not None:
            # target spike count and rates
            self.target_spikes_count = self.get_spikes_count(self.spikes)
            self.target_spikes_rate = self.get_spikes_rate(self.spikes)
        self.initialize(**params)

    def step(self):
        """
        Return the current time step
        """
        return int(self.clock.t/self.dt)
    
    def get_spikes_count(self, spikes):
        count = zeros(self.K)
        for (i,t) in spikes:
            count[i] += 1
        return count
    
    def get_spikes_rate(self, spikes):
        count = self.get_spikes_count(spikes)
        return count*1.0/self.duration
    
    def initialize(self): # TO IMPLEMENT
        """
        Override this method to initialize the criterion before the simulation
        """
        pass
    
    def timestep_call(self): # TO IMPLEMENT
        """
        Override this method to do something at every time step
        """
        pass
    
    def spike_call(self, neurons): # TO IMPLEMENT
        """
        Override this method to do something at every time spike. neurons contains
        the list of neurons that just spiked
        """
        pass
    
    def get_value(self, varname):
        return self.group.state_(varname)
    
    def __call__(self):
        self.timestep_call()
        
    def propagate(self, neurons):
        self.spike_call(neurons)

    def get_values(self): # TO IMPLEMENT
        """
        Override this method to return the criterion values at the end.
        It must return one (or several, as a tuple) additive values, i.e.,
        values corresponding to slices. The method normalize can normalize
        combined values.
        """
        pass
    additive_values = property(get_values)

    def normalize(self, values): # TO IMPLEMENT
        """
        Values contains combined criterion values. It is either a vector of values (as many values
        as neurons), or a tuple with different vector of as many values as neurons.
        """
        pass

class CriterionStruct(object):
    type = None # 'trace', 'spike' or 'both'
    def get_name(self):
        return self.__class__.__name__
    name = property(get_name)




class LpErrorCriterion(Criterion):
    def initialize(self, p=2, varname='v'):
        self.p = p
        self.varname = varname
        self._error = zeros((self.K, self.N))
    
    def timestep_call(self):
        v = self.get_value(self.varname)
        t = self.step()
        if t<self.onset: return # onset
        d = self.intdelays
        indices = (t-d>=0)&(t-d<self.total_steps) # neurons with valid delays (stay inside the target trace)
        vtar = self.traces[:,t-d] # target value at this timestep for every neuron
        for i in xrange(self.K):
            self._error[i,indices] += abs(v[indices]-vtar[i,indices])**self.p
    
    def get_values(self):
        if self.K == 1: error = self._error.flatten()
        else: error = self._error
        return error # just the integral, for every slice
    
    def normalize(self, error):
        # error is now the combined error on the whole duration (sum on the slices)
        # HACK: 1- because modelfitting MAXIMIZES for now...
        return 1-self.dt*error**(1./self.p)

class LpError(CriterionStruct):
    """
    Structure used by the users to specify a criterion
    """
    def __init__(self, p = 2, varname = 'v'):
        self.type = 'trace'
        self.p = p
        self.varname = varname




class GammaFactorCriterion(Criterion):
    """
    Coincidence counter class.
    
    Counts the number of coincidences between the spikes of the neurons in the network (model spikes),
    and some user-specified data spike trains (target spikes). This number is defined as the number of 
    target spikes such that there is at least one model spike within +- ``delta``, where ``delta``
    is the half-width of the time window.
    
    Initialised as::
    
        cc = CoincidenceCounter(source, data, delta = 4*ms)
    
    with the following arguments:
    
    ``source``
        A :class:`NeuronGroup` object which neurons are being monitored.
    
    ``data``
        The list of spike times. Several spike trains can be passed in the following way.
        Define a single 1D array ``data`` which contains all the target spike times one after the
        other. Now define an array ``spiketimes_offset`` of integers so that neuron ``i`` should 
        be linked to target train: ``data[spiketimes_offset[i]], data[spiketimes_offset[i]+1]``, etc.
        
        It is essential that each spike train with the spiketimes array should begin with a spike at a
        large negative time (e.g. -1*second) and end with a spike that is a long time
        after the duration of the run (e.g. duration+1*second).
    
    ``delta=4*ms``
        The half-width of the time window for the coincidence counting algorithm.
    
    ``spiketimes_offset``
        A 1D array, ``spiketimes_offset[i]`` is the index of the first spike of 
        the target train associated to neuron i.
        
    ``spikedelays``
        A 1D array with spike delays for each neuron. All spikes from the target 
        train associated to neuron i are shifted by ``spikedelays[i]``.
        
    ``coincidence_count_algorithm``
        If set to ``'exclusive'``, the algorithm cannot count more than one
        coincidence for each model spike.
        If set to ``'inclusive'``, the algorithm can count several coincidences
        for a single model spike.
    
    ``onset``
        A scalar value in seconds giving the start of the counting: no
        coincidences are counted before ``onset``.
    
    Has three attributes:
    
    ``coincidences``
        The number of coincidences for each neuron of the :class:`NeuronGroup`.
        ``coincidences[i]`` is the number of coincidences for neuron i.
        
    ``model_length``
        The number of spikes for each neuron. ``model_length[i]`` is the spike
        count for neuron i.
        
    ``target_length``
        The number of spikes in the target spike train associated to each neuron.
    """
    def initialize(self, delta, coincidence_count_algorithm):
        self.coincidence_count_algorithm = coincidence_count_algorithm
        self.delta = int(rint(delta / self.dt))
        
        self.spike_count = zeros(self.N, dtype='int')
        self.coincidences = zeros(self.N, dtype='int')
        self.spiketime_index = self.spikes_offset
        self.last_spike_time = array(rint(self.spikes_inline[self.spiketime_index] / self.dt), dtype=int)
        self.next_spike_time = array(rint(self.spikes_inline[self.spiketime_index + 1] / self.dt), dtype=int)

        # First target spikes (needed for the computation of 
        #   the target train firing rates)
#        self.first_target_spike = zeros(self.N)

        self.last_spike_allowed = ones(self.N, dtype='bool')
        self.next_spike_allowed = ones(self.N, dtype='bool')
        
    def spike_call(self, spiking_neurons):
        dt = self.dt
        t = self.step()*dt
        
        spiking_neurons = array(spiking_neurons)
        if len(spiking_neurons):
            
            if t >= self.onset:
                self.spike_count[spiking_neurons] += 1

            T_spiking = array(rint((t + self.delays[spiking_neurons]) / dt), dtype=int)

            remaining_neurons = spiking_neurons
            remaining_T_spiking = T_spiking
            while True:
                remaining_indices, = (remaining_T_spiking > self.next_spike_time[remaining_neurons]).nonzero()
                if len(remaining_indices):
                    indices = remaining_neurons[remaining_indices]
                    self.spiketime_index[indices] += 1
                    self.last_spike_time[indices] = self.next_spike_time[indices]
                    self.next_spike_time[indices] = array(rint(self.spikes_inline[self.spiketime_index[indices] + 1] / dt), dtype=int)
                    if self.coincidence_count_algorithm == 'exclusive':
                        self.last_spike_allowed[indices] = self.next_spike_allowed[indices]
                        self.next_spike_allowed[indices] = True
                    remaining_neurons = remaining_neurons[remaining_indices]
                    remaining_T_spiking = remaining_T_spiking[remaining_indices]
                else:
                    break

            # Updates coincidences count
            near_last_spike = self.last_spike_time[spiking_neurons] + self.delta >= T_spiking
            near_next_spike = self.next_spike_time[spiking_neurons] - self.delta <= T_spiking
            last_spike_allowed = self.last_spike_allowed[spiking_neurons]
            next_spike_allowed = self.next_spike_allowed[spiking_neurons]
            I = (near_last_spike & last_spike_allowed) | (near_next_spike & next_spike_allowed)

            if t >= self.onset:
                self.coincidences[spiking_neurons[I]] += 1

            if self.coincidence_count_algorithm == 'exclusive':
                near_both_allowed = (near_last_spike & last_spike_allowed) & (near_next_spike & next_spike_allowed)
                self.last_spike_allowed[spiking_neurons] = last_spike_allowed & -near_last_spike
                self.next_spike_allowed[spiking_neurons] = (next_spike_allowed & -near_next_spike) | near_both_allowed
            
    def get_values(self):
        return (self.coincidences, self.spike_count)
    
    def normalize(self, values):
        coincidence_count = values[0]
        spike_count = values[1]
        delta = self.delta*self.dt

        gamma = get_gamma_factor(coincidence_count, spike_count, 
                                 self.target_spikes_count, self.target_spikes_rate, 
                                 delta)
        
        return gamma

class GammaFactor(CriterionStruct):
    def __init__(self, delta = 4*ms, coincidence_count_algorithm = 'exclusive'):
        self.type = 'spikes'
        self.delta = delta
        self.coincidence_count_algorithm = coincidence_count_algorithm




class VanRossumCriterion(Criterion):
    def initialize(self, tau):
        self.delay_range =max(self.delays)- min(self.delays)#delay range
        self.min_delay = abs(min(self.delays))#minimum of possible delay
        self.distance_vector=zeros(self.N) 
        self.nbr_neurons_group = self.N/self.K
        
        eqs="""
        dv/dt=(-v)/tau: volt
        """
        # network to convolve target spikes with the kernel
        self.input_target=SpikeGeneratorGroup(self.K,self.spikes,clock=self.group.clock)
        self.kernel_target=NeuronGroup(self.K,model=eqs,clock=self.group.clock)
        self.C_target = DelayConnection(self.input_target, self.kernel_target, 'v', structure='dense',  max_delay=self.min_delay)     
        self.C_target.connect_one_to_one(self.input_target,self.kernel_target)
        self.C_target.delay = self.min_delay*ones_like(self.C_target.delay)

        # network to convolve population spikes with the kernel
        self.kernel_population=NeuronGroup(self.N,model=eqs,clock=self.group.clock)
        self.C_population = DelayConnection(self.group, self.kernel_population, 'v', structure='sparse',  max_delay=self.delay_range)
        for iN in xrange(self.N):
            self.C_population.delay[iN,iN] = diagflat(self.min_delay + self.delays[iN])
        self.C_population.connect_one_to_one(self.group,self.kernel_population)
        self.spikecount = SpikeCounter(self.group)
        self.contained_objects = [self.kernel_population,self.C_population,self.spikecount,self.input_target,self.C_target,self.kernel_target]  

    def __call__(self):
        trace_population = self.kernel_population.state_('v')
        trace_target = self.kernel_target.state_('v')
        for igroup in xrange(self.K):
            self.distance_vector[igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group] += (trace_population[igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group]-trace_target[igroup])**2

    def get_values(self):
        return (self.distance_vector)
    
    def normalize(self, distance_vector):
        distance_vector[nonzero(self.spikecount.count==0)] = inf
        return -self.distance_vector*self.group.clock.dt


class VanRossum(CriterionStruct):
    def __init__(self, tau):
        self.type = 'spikes'
        self.tau = tau


class BretteCriterion(Criterion):
    def initialize(self,tau_metric):
        self.delay_range =max(self.delays)- min(self.delays)#delay range
        self.min_delay = abs(min(self.delays))#minimum of possible delay
        self.corr_vector=zeros(self.N) 
        self.norm_pop = zeros(self.N) 
        self.norm_target = zeros(self.N) 
        self.nbr_neurons_group = self.N/self.K
        
        eqs="""
        tau:second
        dv/dt=(-v)/tau: volt
        """
        # network to convolve target spikes with the kernel
        self.input_target=SpikeGeneratorGroup(self.K,self.spikes,clock=self.group.clock)
        self.kernel_target=NeuronGroup(self.N,model=eqs,clock=self.group.clock)
        self.C_target = DelayConnection(self.input_target, self.kernel_target, 'v', structure='sparse',  max_delay=self.min_delay)  
        self.kernel_target.tau=tau_metric
        for igroup in xrange(self.K):
            self.C_target.W[igroup,igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group] = ones(self.nbr_neurons_group)
            self.C_target.delay[igroup,igroup*self.nbr_neurons_group:(1+igroup)*self.nbr_neurons_group] =  self.min_delay * ones(self.nbr_neurons_group)
            
        # network to convolve population spikes with the kernel
        self.kernel_population=NeuronGroup(self.N,model=eqs,clock=self.group.clock)
        self.C_population = DelayConnection(self.group, self.kernel_population, 'v', structure='sparse',  max_delay=self.delay_range)
        for iN in xrange(self.N):
            self.C_population.delay[iN,iN] = diagflat(self.min_delay + self.delays[iN])
        self.C_population.connect_one_to_one(self.group,self.kernel_population)
        self.kernel_population.tau=tau_metric
        self.spikecount = SpikeCounter(self.group)
        self.contained_objects = [self.kernel_population,self.C_population,self.spikecount,self.input_target,self.C_target,self.kernel_target]  

    def __call__(self):
        trace_population = self.kernel_population.state_('v')
        trace_target = self.kernel_target.state_('v')
        self.corr_vector += trace_population*trace_target
        self.norm_pop += trace_population**2
        self.norm_target += trace_target**2
        
    def get_values(self):
        return (self.corr_vector,self.norm_pop,self.norm_target)
    
    def normalize(self, values):
        corr_vector=values[0]
        norm_pop=values[1]
        norm_target=values[2]
        corr_vector[nonzero(self.spikecount.count==0)] = -inf
        #print self.corr_vector/sqrt(norm_pop)/sqrt(norm_target)
        return self.corr_vector/sqrt(norm_pop)/sqrt(norm_target)


class Brette(CriterionStruct):
    def __init__(self):
        self.type = 'spikes'



class ModelFitting(Fitness):
    def initialize(self, **kwds):
        # Initialization of variables
        self.use_gpu = self.unit_type=='GPU'
        # Gets the key,value pairs in shared_data
        for key, val in self.shared_data.iteritems():
            setattr(self, key, val)
        # Gets the key,value pairs in **kwds
        for key, val in kwds.iteritems():
            setattr(self, key, val)
        self.neurons = self.nodesize
        self.groups = self.groups
        self.model = cPickle.loads(self.model)
        if type(self.model) is str:
            self.model = Equations(self.model)
        
        self.initialize_neurongroup()
        self.transform_data()
        self.inject_input()
        
#        if self.use_gpu:
#            ########
#            # TODO
#            ########
#            # Select integration scheme according to method
#            if self.method == 'Euler': scheme = euler_scheme
#            elif self.method == 'RK': scheme = rk2_scheme
#            elif self.method == 'exponential_Euler': scheme = exp_euler_scheme
#            else: raise Exception("The numerical integration method is not valid")
#            
#            self.mf = GPUModelFitting(self.group, self.model, self.input, self.I_offset,
#                                      self.spiketimes, self.spiketimes_offset, zeros(self.neurons), 0*ms, self.delta,
#                                      precision=self.precision, scheme=scheme)
#        else:
#            self.cc = CoincidenceCounter(self.group, self.spiketimes, self.spiketimes_offset,
#                                        onset=self.onset, delta=self.delta)
    
    def initialize_neurongroup(self):
        # Add 'refractory' parameter on the CPU only
        if not self.use_gpu:
            if self.max_refractory is not None:
                refractory = 'refractory'
                self.model.add_param('refractory', second)
            else:
                refractory = self.refractory
        else:
            if self.max_refractory is not None:
                refractory = 0*ms
            else:
                refractory = self.refractory

        # Must recompile the Equations : the functions are not transfered after pickling/unpickling
        self.model.compile_functions()

        self.group = NeuronGroup(self.neurons,
                                 model=self.model,
                                 reset=self.reset,
                                 threshold=self.threshold,
                                 refractory=refractory,
                                 max_refractory = self.max_refractory,
                                 method = self.method,
                                 clock=Clock(dt=self.dt))
        
        if self.initial_values is not None:
            for param, value in self.initial_values.iteritems():
                self.group.state(param)[:] = value
    
    def transform_data(self):
        self.transformer = DataTransformer(self.neurons,
                                           self.inputs,
                                           spikes = self.spikes, 
                                           traces = self.traces,
                                           dt = self.dt,
                                           slices = self.slices,
                                           overlap = self.overlap, 
                                           groups = self.groups)
        self.total_steps = self.transformer.total_steps
        self.sliced_duration = self.transformer.sliced_duration
        
        self.sliced_inputs = self.transformer.slice_traces(self.inputs)
        self.inputs_inline, self.inputs_offset = self.transformer.transform_traces(self.sliced_inputs)
        
        if self.traces is not None:
            self.sliced_traces = self.transformer.slice_traces(self.traces)
            self.traces_inline, self.traces_offset = self.transformer.transform_traces(self.sliced_traces)
        else:
            self.sliced_traces, self.traces_inline, self.traces_offset = None, None, None
        
        if self.spikes is not None:
            self.sliced_spikes = self.transformer.slice_spikes(self.spikes)
            self.spikes_inline, self.spikes_offset = self.transformer.transform_spikes(self.sliced_spikes)
        else:
            self.sliced_spikes, self.spikes_inline, self.spikes_offset = None, None, None
    
    def inject_input(self):
        # Injects current in consecutive subgroups, where I_offset have the same value
        # on successive intervals
        I_offset = self.inputs_offset
        k = -1
        for i in hstack((nonzero(diff(I_offset))[0], len(I_offset) - 1)):
            I_offset_subgroup_value = I_offset[i]
            I_offset_subgroup_length = i - k
            sliced_subgroup = self.group.subgroup(I_offset_subgroup_length)
            input_sliced_values = self.inputs_inline[I_offset_subgroup_value:I_offset_subgroup_value + self.total_steps]
            sliced_subgroup.set_var_by_array(self.input_var, TimedArray(input_sliced_values, clock=self.group.clock))
            k = i
    
    def initialize_criterion(self, delays,tau_metric = None):
        # general criterion parameters
        params = dict(group=self.group, traces=self.sliced_traces, spikes=self.sliced_spikes, 
                      targets_count=self.groups*self.slices, duration=self.sliced_duration, onset=self.onset, 
                      spikes_inline=self.spikes_inline, spikes_offset=self.spikes_offset,
                      traces_inline=self.traces_inline, traces_offset=self.traces_offset,
                      delays=delays, when='start')
        
        criterion_name = self.criterion.__class__.__name__
        
        # criterion-specific parameters
        if criterion_name == 'GammaFactor':
            params['delta'] = self.criterion.delta
            params['coincidence_count_algorithm'] = self.criterion.coincidence_count_algorithm
            self.criterion_object = GammaFactorCriterion(**params)
            
        if criterion_name == 'LpError':
            params['p'] = self.criterion.p
            params['varname'] = self.criterion.varname
            self.criterion_object = LpErrorCriterion(**params)
            
        if criterion_name == 'VanRossum':
            params['tau'] = self.criterion.tau
            self.criterion_object = VanRossumCriterion(**params)
            
        if criterion_name == 'Brette':
            params['tau_metric'] = tau_metric
            self.criterion_object = BretteCriterion(**params)

    
    def update_neurongroup(self, **param_values):
        """
        Inject fitting parameters into the NeuronGroup
        """
        # Sets the parameter values in the NeuronGroup object
        self.group.reinit()
        for param, value in param_values.iteritems():
            self.group.state(param)[:] = kron(value, ones(self.slices)) # kron param_values if slicing
        
        # Reinitializes the model variables
        if self.initial_values is not None:
            for param, value in self.initial_values.iteritems():
                self.group.state(param)[:] = value
    
    def combine_sliced_values(self, values):
        if type(values) is tuple:
            combined_values = tuple([sum(reshape(v, (self.slices, -1)), axis=0) for v in values])
        else:
            combined_values = sum(reshape(values, (self.slices, -1)), axis=0)
        return combined_values
    
    def evaluate(self, **param_values):
        """
        Use fitparams['delays'] to take delays into account
        Use fitparams['refractory'] to take refractory into account
        """
        delays = param_values.pop('delays', zeros(self.neurons))
        refractory = param_values.pop('refractory', zeros(self.neurons))
        tau_metric = param_values.pop('tau_metric', zeros(self.neurons))

        # repeat spike delays and refractory to take slices into account
        delays = kron(delays, ones(self.slices))
        refractory = kron(refractory, ones(self.slices))
        tau_metric = kron(tau_metric, ones(self.slices))
        
        self.update_neurongroup(**param_values)
        if self.criterion.__class__.__name__ == 'Brette':
            self.initialize_criterion(delays,tau_metric)
        else:
            self.initialize_criterion(delays)
        
        if self.use_gpu:
            pass
            #########
            # TODO
            #########
#            # Reinitializes the simulation object
#            self.mf.reinit_vars(self.input, self.I_offset, self.spiketimes, self.spiketimes_offset, delays, refractory)
#            # LAUNCHES the simulation on the GPU
#            self.mf.launch(self.duration, self.stepsize)
#            coincidence_count = self.mf.coincidence_count
#            spike_count = self.mf.spike_count
        else:
            # set the refractory period
            if self.max_refractory is not None:
                self.group.refractory = refractory
            
            # Launch the simulation on the CPU
            self.group.clock.reinit()
            net = Network(self.group, self.criterion_object)
            net.run(self.duration)
        
        sliced_values = self.criterion_object.get_values()
        combined_values = self.combine_sliced_values(sliced_values)
        values = self.criterion_object.normalize(combined_values)
        return values




def modelfitting(model=None,
                 reset=None,
                 threshold=None,
                 refractory=0*ms,
                 data=None,
                 input_var='I',
                 input=None,
                 dt=None,
                 popsize=1000,
                 maxiter=10,
                 slices=1,
                 overlap=0*second,
                 initial_values=None,
                 stepsize=100 * ms,
                 unit_type='CPU',
                 total_units=None,
                 cpu=None,
                 gpu=None,
                 precision='double', # set to 'float' or 'double' to specify single or double precision on the GPU
                 machines=[],
                 allocation=None,
                 returninfo=False,
                 scaling=None,
                 algorithm=CMAES,
                 criterion=None,
                 optparams={},
                 method='Euler',
                 **params):
    """
    Model fitting function.
    
    Fits a spiking neuron model to electrophysiological data (injected current and spikes).
    
    See also the section :ref:`model-fitting-library` in the user manual.
    
    **Arguments**
    
    ``model``
        An :class:`~brian.Equations` object containing the equations defining the model.
    
    ``reset``
        A reset value for the membrane potential, or a string containing the reset
        equations.
        
    ``threshold``
        A threshold value for the membrane potential, or a string containing the threshold
        equations.
        
    ``refractory``
        The refractory period in second. If it's a single value, the same refractory will be
        used in all the simulations. If it's a list or a tuple, the fitting will also
        optimize the refractory period (see ``**params`` below).
        
        Warning: when using a refractory period, you can't use a custom reset, only a fixed one.
        
    ``data``
        A list of spike times, or a list of several spike trains as a list of pairs (index, spike time)
        if the fit must be performed in parallel over several target spike trains. In this case,
        the modelfitting function returns as many parameters sets as target spike trains.
        
    ``input_var='I'``
        The variable name used in the equations for the input current.
        
    ``input``
        A vector of values containing the time-varying signal the neuron responds to (generally
        an injected current).
        
    ``dt``
        The time step of the input (the inverse of the sampling frequency).
        
    ``**params``
        The list of parameters to fit the model with. Each parameter must be set as follows:
        ``param_name=[bound_min, min, max, bound_max]``
        where ``bound_min`` and ``bound_max`` are the boundaries, and ``min`` and ``max``
        specify the interval from which the parameter values are uniformly sampled at
        the beginning of the optimization algorithm.
        If not using boundaries, set ``param_name=[min, max]``.
        
        Also, you can add a fit parameter which is a spike delay for all spikes :
        add the special parameter ``delays`` in ``**params``, for example 
        ``modelfitting(..., delays=[-10*ms, 10*ms])``.
        
        You can also add fit the refractory period by specifying 
        ``modelfitting(..., refractory=[-10*ms, 10*ms])``.
        
    ``popsize``
        Size of the population (number of particles) per target train used by the optimization algorithm.
        
    ``maxiter``
        Number of iterations in the optimization algorithm.
        
    ``optparams``
        Optimization algorithm parameters. It is a dictionary: keys are parameter names,
        values are parameter values or lists of parameters (one value per group). 
        This argument is specific to the optimization
        algorithm used. See :class:`PSO`, :class:`GA`, :class:`CMAES`. 
        
    ``delta=4*ms``
        The precision factor delta (a scalar value in second).
        
    ``slices=1``
        The number of time slices to use.
        
    ``overlap=0*ms``
        When using several time slices, the overlap between consecutive slices, in seconds.
        
    ``initial_values``
        A dictionary containing the initial values for the state variables.
        
    ``cpu``
        The number of CPUs to use in parallel. It is set to the number of CPUs in the machine by default.
        
    ``gpu``
        The number of GPUs to use in parallel. It is set to the number of GPUs in the machine by default.
        
    ``precision``
        GPU only: a string set to either ``float`` or ``double`` to specify whether to use
        single or double precision on the GPU. If it is not specified, it will
        use the best precision available.
        
    ``returninfo=False``
        Boolean indicating whether the modelfitting function should return technical information
        about the optimization.
        
    ``scaling=None``
        Specify the scaling used for the parameters during the optimization. 
        It can be ``None`` or ``'mapminmax'``. It is ``None``
        by default (no scaling), and ``mapminmax`` by default for the CMAES algorithm.
        
     ``algorithm=CMAES``
         
         
     ``optparams={}``
         Optimization parameters. See
         
    ``method='Euler'``
        Integration scheme used on the CPU and GPU: ``'Euler'`` (default), ``RK``, 
        or ``exponential_Euler``. 
        See also :ref:`numerical-integration`.
        
    ``machines=[]``
        A list of machine names to use in parallel. See :ref:`modelfitting-clusters`.
    
    **Return values**
    
    Return an :class:`OptimizationResult` object with the following attributes:
    
    ``best_pos``
        Minimizing position found by the algorithm. For array-like fitness functions,
        it is a single vector if there is one group, or a list of vectors.
        For keyword-like fitness functions, it is a dictionary
        where keys are parameter names and values are numeric values. If there are several groups,
        it is a list of dictionaries.
    
    ``best_fit``
        The value of the fitness function for the best positions. It is a single value if 
        there is one group, or it is a list if there are several groups.
    
    ``info``
        A dictionary containing various information about the optimization.


    Also, the following syntax is possible with an ``OptimizationResult`` instance ``or``.
    The ``key`` is either an optimizing parameter name for keyword-like fitness functions,
    or a dimension index for array-like fitness functions.
    
    ``or[key]``
        it is the best ``key`` parameter found (single value), or the list
        of the best parameters ``key`` found for all groups.
    
    ``or[i]``
        where ``i`` is a group index. This object has attributes ``best_pos``, ``best_fit``,
        ``info`` but only for group ``i``.
    
    ``or[i][key]``
        where ``i`` is a group index, is the same as ``or[i].best_pos[key]``.

    For more details on the gamma factor, see
    `Jolivet et al. 2008, "A benchmark test for a quantitative assessment of simple neuron models", J. Neurosci. Methods <http://www.ncbi.nlm.nih.gov/pubmed/18160135>`__ (available in PDF
    `here <http://icwww.epfl.ch/~gerstner/PUBLICATIONS/Jolivet08.pdf>`__).
    """

    if criterion is None:
        criterion = GammaFactor()
        
    data = array(data)
    if criterion.type == 'spikes':
        # Make sure that 'data' is a N*2-array
        if data.ndim == 1:
            data = concatenate((zeros((len(data), 1)), data.reshape((-1, 1))), axis=1)
        spikes = data
        traces = None 
    elif criterion.type == 'trace':
        if data.ndim == 1:
            data = data.reshape((1,-1))
        spikes = None
        traces = data
    elif criterion.type == 'both':
        # TODO
        log_warn("Not implemented yet")
        pass
    inputs = input
    if inputs.ndim==1:
        inputs = inputs.reshape((1,-1))

    # dt must be set
    if dt is None:
        raise Exception('dt (sampling frequency of the input) must be set')

    # default overlap when no time slicing
    if slices == 1:
        overlap = 0*ms

    # default allocation
    if cpu is None and gpu is None:
        if CANUSEGPU: gpu = 1
        else: cpu = MAXCPU-1

    # check numerical integration method
    if (gpu>0 or unit_type == 'GPU') and method not in ['Euler', 'RK', 'exponential_Euler']:
        raise Exception("The method can only be 'Euler', 'RK', or 'exponential_Euler' when using the GPU") 
    if method not in ['Euler', 'RK', 'exponential_Euler', 'linear', 'nonlinear']:
        raise Exception("The method can only be 'Euler', 'RK', 'exponential_Euler', 'linear', or 'nonlinear'")

    if (algorithm == CMAES) & (scaling is None):
        scaling = 'mapminmax'
        
    # determines whether optimization over refractoriness or not
    if type(refractory) is tuple or type(refractory) is list:
        params['refractory'] = refractory
        max_refractory = refractory[-1]
#        refractory = 'refractory'
    else:
        max_refractory = None

    # common values
#    group_size = particles # Number of particles per target train
    groups = int(array(data)[:, 0].max() + 1) # number of target trains
#    N = group_size * group_count # number of neurons
    duration = len(input) * dt # duration of the input

    # keyword arguments for Modelfitting initialize
    kwds = dict(   model=cPickle.dumps(model),
                   threshold=threshold,
                   reset=reset,
                   refractory=refractory,
                   max_refractory=max_refractory,
                   input_var=input_var, dt=dt,
                   duration=duration,
                   criterion=criterion,
                   slices=slices,
                   overlap=overlap,
                   returninfo=returninfo,
                   precision=precision,
                   stepsize=stepsize,
                   method=method,
                   onset=overlap)

    shared_data = dict(inputs=inputs,
                       traces=traces,
                       spikes=spikes,
                       initial_values=initial_values)

    r = maximize(   ModelFitting,
                    shared_data=shared_data,
                    kwds = kwds,
                    groups=groups,
                    popsize=popsize,
                    maxiter=maxiter,
                    optparams=optparams,
                    unit_type = unit_type,
                    machines=machines,
                    allocation=allocation,
                    cpu=cpu,
                    gpu=gpu,
                    returninfo=returninfo,
                    codedependencies=[],
                    algorithm=algorithm,
                    scaling=scaling,
                    **params)

    # r is (results, fitinfo) or (results)
    return r




def get_spikes(model=None, reset=None, threshold=None,
                input=None, input_var='I', dt=None,
                **params):
    """
    Retrieves the spike times corresponding to the best parameters found by
    the modelfitting function.
    
    **Arguments**
    
    ``model``, ``reset``, ``threshold``, ``input``, ``input_var``, ``dt``
        Same parameters as for the ``modelfitting`` function.
        
    ``**params``
        The best parameters returned by the ``modelfitting`` function.
    
    **Returns**
    
    ``spiketimes``
        The spike times of the model with the given input and parameters.
    """
    duration = len(input) * dt
    ngroups = len(params[params.keys()[0]])

    group = NeuronGroup(N=ngroups, model=model, reset=reset, threshold=threshold,
                        clock=Clock(dt=dt))
    group.set_var_by_array(input_var, TimedArray(input, clock=group.clock))
    for param, values in params.iteritems():
        if (param == 'delays') | (param == 'fitness'):
            continue
        group.state(param)[:] = values

    M = SpikeMonitor(group)
    net = Network(group, M)
    net.run(duration)
    reinit_default_clock()
    return M.spikes

def predict(model=None, reset=None, threshold=None,
            data=None, delta=4 * ms,
            input=None, input_var='I', dt=None,
            **params):
    """
    Predicts the gamma factor of a fitted model with respect to the data with
    a different input current.
    
    **Arguments**
    
    ``model``, ``reset``, ``threshold``, ``input_var``, ``dt``
        Same parameters as for the ``modelfitting`` function.
        
    ``input``
        The input current, that can be different from the current used for the fitting
        procedure.
    
    ``data``
        The experimental spike times to compute the gamma factor against. They have
        been obtained with the current ``input``.
    
    ``**params``
        The best parameters returned by the ``modelfitting`` function.
    
    **Returns**
    
    ``gamma``
        The gamma factor of the model spike trains against the data.
        If there were several groups in the fitting procedure, it is a vector
        containing the gamma factor for each group.
    """
    spikes = get_spikes(model=model, reset=reset, threshold=threshold,
                        input=input, input_var=input_var, dt=dt,
                        **params)

    ngroups = len(params[params.keys()[0]])
    gamma = zeros(ngroups)
    for i in xrange(ngroups):
        spk = [t for j, t in spikes if j == i]
        gamma[i] = gamma_factor(spk, data, delta, normalize=True, dt=dt)
    if len(gamma) == 1:
        return gamma[0]
    else:
        return gamma



if __name__ == '__main__':
    
    from brian import loadtxt, ms, savetxt, loadtxt, Equations, NeuronGroup, run, SpikeMonitor,\
         StateMonitor, Network
    from pylab import *
    
    def generate_data():
        g = NeuronGroup(1, model=equations, reset=0, threshold=1)
        g.I = TimedArray(input, dt=.1*ms)
        g.tau = 25*ms
        g.R = 3e9
        SpM = SpikeMonitor(g)
        StM = StateMonitor(g, 'V', record=True)
        net = Network(g, SpM, StM)
        net.run(1*second)
        return StM.values[0], SpM.spikes
    
    equations = Equations('''
        dV/dt=(R*I-V)/tau : 1
        I : 1
        R : 1
        tau : second
    ''')
    input = loadtxt('current.txt')
    
    # ARTIFICIAL DATA: R=3e9, tau=25*ms
#    spikes = loadtxt('spikes.txt') # real data
#    trace, spikes = generate_data()
#    savetxt('trace_artificial.txt', trace)
#    savetxt('spikes_artificial.txt', spikes)
    
    trace = loadtxt('trace_artificial.txt')
    spikes= loadtxt('spikes_artificial.txt')
    
    # GAMMA FACTOR
#    criterion = GammaFactor(delta=4*ms)
#    data = spikes
    
    # LP ERROR
#    criterion = LpError(p=2, varname='V')
#    data = trace
    
     ##Van Rossum ERROR
#    criterion = VanRossum(tau=4*ms)
#    data = spikes
    
    
    
    equations = Equations('''
    dV/dt=(R*I-V)/tau : 1
    I : 1
    R : 1
    tau_metric:second
    tau : second
    ''')
    # Brette ERROR
    criterion = Brette()
    data = spikes
#    
    results = modelfitting( model = equations,
                            reset = 0,
                            threshold = 1,
                            data = data,
                            input = input,
                            cpu = 1,
                            dt = .1*ms,
                            popsize = 1000,
                            maxiter = 5,
                            criterion = criterion,
                            R = [1.0e9, 9.0e9],
                            tau = [10*ms, 40*ms],
                            tau_metric=[1*ms,10*ms]
#                            delays = [-5*ms, 5*ms],
#                            refractory = [0*ms, 0*ms, 10*ms, 10*ms]
                            )
    
    
    print_table(results)
    
    
    


'''
Spike-timing dependent plasticity
Adapted from Song, Miller and Abbott (2000) and Song and Abbott (2001)

This simulation takes a long time!
'''
from brian import *
from time import time
from eventbased_stdp import *

#log_level_debug() # uncomment to see the STDP equations

N = 1000
taum = 10 * ms
tau_pre = 20 * ms
tau_post = tau_pre
Ee = 0 * mV
vt = -54 * mV
vr = -60 * mV
El = -74 * mV
taue = 5 * ms
F = 15 * Hz
gmax = .01
dA_pre = .01
dA_post = -dA_pre * tau_pre / tau_post * 1.05

eqs_neurons = '''
dv/dt=(ge*(Ee-vr)+El-v)/taum : volt   # the synaptic current is linearized
dge/dt=-ge/taue : 1
'''

input = PoissonGroup(N, rates=F)
neurons = NeuronGroup(1, model=eqs_neurons, threshold=vt, reset=vr)
synapses = Connection(input, neurons, 'ge', weight=rand(len(input), len(neurons)) * gmax)#, structure='dense')
neurons.v = vr

## Explicit STDP rule
eqs_stdp = '''
A_pre : 1
A_post : 1
last_t : second # last update
'''
dA_post *= gmax
dA_pre *= gmax
pre = """
A_pre=A_pre*exp(-(t-last_t)/tau_pre)+dA_pre
A_post=A_post*exp(-(t-last_t)/tau_post)
w+=A_post
last_t = t
"""
post = """
A_pre=A_pre*exp(-(t-last_t)/tau_pre)
A_post=A_post*exp(-(t-last_t)/tau_post)+dA_post
w+=A_pre
last_t = t
"""
stdp = EventBasedSTDP(synapses, eqs=eqs_stdp, pre=pre,post=post, wmax=gmax)

rate = PopulationRateMonitor(neurons)

start_time = time()

run(100 * second, report='text')
print "Simulation time:", time() - start_time

figure()
subplot(311)
plot(rate.times / second, rate.smooth_rate(30 * ms))
subplot(312)
plot(synapses.W.todense() / gmax, '.')
subplot(313)
hist(synapses.W.todense() / gmax, 20)
show()



'''
Ideas for automating the work of parallelising a network

* Given a network object net, work through each of the objects
  adapting them all for the expansion of some parameter. This
  is nice because it doesn't require the user to write any new
  code just add in an extra auto-parallelisation line. The
  difficult thing is that it would require adapting the objects
  so that as well as creating them they also store a copy of
  their parameters in such a way that they can be recreated
  (e.g. extending a neurongroup from N=1 to N=1000). This is
  tricky at the moment.
* Given a function that defines a simulation to be run, rewrite
  the code automatically. One nice way to do this is to replace
  each NeuronGroup object in the global namespace of the function
  with a new type of NeuronGroup object that automatically adapts
  it to the parameter set given (multiplies N by number of
  different parameters, etc.). This is also nice since it means
  the user just has to wrap their code in a function. The
  difficult thing here is that referring to the objects
  defined by the function is tricky - how should it be done?
'''

from brian import *
import inspect
from copy import copy

def printargs(*args, **kwds):
    print args
    print kwds


class AutoParallelNetwork(Network):
    def __init__(self, makenet, params):
        self.apn_params = params
        self.apn_makenet = makenet
        Network.__init__(self)

        makenet_globals = copy(makenet.func_globals)
        makenet_globals['NeuronGroup'] = printargs
        newfuncspace = {}
        exec inspect.getsource(makenet) in makenet_globals, newfuncspace
        exec makenet.func_name + '()' in makenet_globals, newfuncspace

def makenet():
    global M
    Vr = -70 * mV
    Vt = -55 * mV
    El = -54 * mV
    eqs = '''
        dV/dt = -(V-El)/tau : volt
        tau : second
        '''
    model = Model(equations=eqs, threshold=Vt, reset=Vr)
    G = NeuronGroup(1, model)
#    spikes = linspace(10*ms,100*ms,25)
#    input = MultipleSpikeGeneratorGroup([spikes])
#    C = Connection(input, G)
#    C[0,0] = 5*mV
    M = StateMonitor(G, 'V', record=True)
#    G.V = Vr
#    G.tau = 10*ms
#    net = MagicNetwork(verbose=False)
#    return net

makenet()

print 'made net'

anet = AutoParallelNetwork(makenet,
        [
         ('G', 'tau', [5 * ms, 10 * ms, 15 * ms])
        ]
        )

#net = makenet()
#net.run(100*ms)
#
#plot(M.times/ms,M[0]/mV)
#show()



# NON-ATOMIC ALGORITHM STILL DOESN'T WORK IF N>16K,
# ALMOST CERTAINLY TO DO WITH SHARED MEMORY SIZE
import pycuda.autoinit as autoinit
import pycuda.driver as drv
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
import bisect
import numpy, pylab, time, random
from scipy import weave

N = 512 * 512
nspike = int(0.1 * N)
blocksize = 512
duration = 10000
use_atomic_threshold = False

N = int(numpy.ceil(1. * N / blocksize) * blocksize)

if drv.get_version() == (2, 0, 0): # cuda version
    precision = 'float'
elif drv.get_version() > (2, 0, 0):
    precision = 'double'
else:
    raise Exception, "CUDA 2.0 required"

if precision == 'double':
    mydtype = numpy.float64
else:
    mydtype = numpy.float32

block = (blocksize, 1, 1)
Ngrid = N / blocksize
grid = (Ngrid, 1)
grid1 = (1, 1)

mod = drv.SourceModule("""
__global__ void stateupdate(SCALAR *V_arr, SCALAR *ge_arr, SCALAR *gi_arr)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    SCALAR V = V_arr[i];
    SCALAR ge = ge_arr[i];
    SCALAR gi = gi_arr[i]; 
    SCALAR V__tmp = (ge+gi-(V+0.049))/0.02;
    SCALAR ge__tmp = -ge/0.005;
    SCALAR gi__tmp = -gi/0.01;
    V_arr[i] = V+0.0001*V__tmp;
    ge_arr[i] = ge+0.0001*ge__tmp;
    gi_arr[i] = gi+0.0001*gi__tmp;
}

__global__ void threshold(SCALAR *V, int *spikes, bool *spiked, unsigned int *global_j, int N)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    bool this_spiked = V[i]>-0.05; 
    spiked[i] = this_spiked;
    if(this_spiked)
    {
        spikes[atomicInc(global_j, N)] = i;
    }
}

/////// TODO: next three functions are untested, but should be a non-blocking threshold function if N<=512*512

__global__ void threshold_blocksumcount(SCALAR *V, unsigned int *blocksumcount)
{ 
    __shared__ unsigned short int partialsum[BLOCKSIZE];
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    int t = threadIdx.x;
    int b = blockIdx.x;
    bool this_spiked = V[i]>-0.05;
    partialsum[t] = (unsigned short int)this_spiked;
    for(unsigned int stride=blockDim.x/2; stride>=1; stride/=2)
    {
        __syncthreads();
        if(t<stride)
            partialsum[t] += partialsum[t+stride];
    }
    __syncthreads();
    if(t==0)
        blocksumcount[b] = partialsum[t];
}

__global__ void threshold_cumsum(unsigned int *blocksumcount, unsigned int *cumblocksumcount)
{
    __shared__ unsigned int partialsum[BLOCKSIZE];
    int t = threadIdx.x;
    partialsum[t] = blocksumcount[t];
    for(unsigned int stride=1; stride<blockDim.x; stride*=2)
    {
        __syncthreads();
        if(t>=stride)
            partialsum[t] += partialsum[t-stride];
    }
    cumblocksumcount[t+1] = partialsum[t];
}

__global__ void threshold_compact(SCALAR *V, int *spikes, unsigned int *blocksumcount, unsigned int *cumblocksumcount)
{
    __shared__ unsigned short int partialsum[BLOCKSIZE];
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    int t = threadIdx.x;
    int b = blockIdx.x;
    // only compact those blocks with some spikes in (many will not in general, and this introduces no divergence)
    if(blocksumcount[b]>0)
    {
        bool this_spiked = V[i]>-0.05;
        partialsum[t] = (unsigned short int)this_spiked;
        for(unsigned int stride=1; stride<blockDim.x; stride*=2)
        {
            __syncthreads();
            if(t>=stride)
                partialsum[t] += partialsum[t-stride];
        }
        __syncthreads();
        if(this_spiked)
          spikes[(unsigned int)partialsum[t]+cumblocksumcount[b]-1] = i;
        //spikes[i] = partialsum[t]+cumblocksumcount[b];
    }
}
""".replace('SCALAR', precision).replace('BLOCKSIZE', str(blocksize)))
stateupdate = mod.get_function("stateupdate")
threshold = mod.get_function("threshold")
threshold_blocksumcount = mod.get_function("threshold_blocksumcount")
threshold_cumsum = mod.get_function("threshold_cumsum")
threshold_compact = mod.get_function("threshold_compact")

V_cpu = numpy.zeros(N, dtype=mydtype)
V_cpu[:nspike] = -0.04 # above threshold
V_cpu[nspike:] = -0.06 # below threshold
random.shuffle(V_cpu)
V = gpuarray.to_gpu(V_cpu)
ge = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))
gi = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))

gpu_spikes = drv.mem_alloc(4 * N)
gpu_spiked = gpuarray.to_gpu(numpy.zeros(N, dtype=bool))
gpu_spike_index = drv.mem_alloc(4)
spikes = master_spikes = numpy.zeros(N, dtype=int)
spike_index = numpy.zeros(1, dtype=numpy.uint32)
gpu_blocksumcount = gpuarray.to_gpu(numpy.zeros(Ngrid, dtype=numpy.uint32))
gpu_cumblocksumcount = gpuarray.to_gpu(numpy.zeros(Ngrid + 1, dtype=numpy.uint32))
drv.memcpy_htod(gpu_spikes, master_spikes)
drv.memcpy_htod(gpu_spike_index, spike_index)

stateupdate.prepare(('i', 'i', 'i'), block)
threshold.prepare(('i', 'i', 'i', 'i', numpy.int32), block)
threshold_blocksumcount.prepare(('i', 'i'), block)
threshold_cumsum.prepare(('i', 'i'), block)
threshold_compact.prepare(('i', 'i', 'i', 'i'), block)

stateupdate_args = (grid, int(V.gpudata), int(ge.gpudata), int(gi.gpudata))
threshold_args = (grid, int(V.gpudata), int(gpu_spikes), int(gpu_spiked.gpudata), int(gpu_spike_index), numpy.int32(N))
threshold_blocksumcount_args = (grid, int(V.gpudata), int(gpu_blocksumcount.gpudata))
threshold_cumsum_args = (grid1, int(gpu_blocksumcount.gpudata), int(gpu_cumblocksumcount.gpudata))
threshold_compact_args = (grid, int(V.gpudata), int(gpu_spikes), int(gpu_blocksumcount.gpudata), int(gpu_cumblocksumcount.gpudata))
stateupdate.prepared_call(*stateupdate_args)
threshold.prepared_call(*threshold_args)

if 0:
    #print 'before blocksumcount'
    threshold_blocksumcount.prepared_call(*threshold_blocksumcount_args)
    autoinit.context.synchronize()
    #print gpu_blocksumcount.get()
    #print sum(V_cpu[:512]>-0.05), sum(V_cpu[512:1024]>-0.05) # these two should be the same...

    #print numpy.cumsum(gpu_blocksumcount.get())
    #print 'before cumsum'
    threshold_cumsum.prepared_call(*threshold_cumsum_args)
    autoinit.context.synchronize()
    #print gpu_cumblocksumcount.get()
    A1 = gpu_cumblocksumcount.get()[1:]
    A2 = numpy.cumsum(gpu_blocksumcount.get())
    print 'Passed first cumsum test:', (A1 == A2).all()
    #print 'before compact'
    threshold_compact.prepared_call(*threshold_compact_args)
    autoinit.context.synchronize()

    drv.memcpy_dtoh(spikes, gpu_spikes)

    #pylab.plot(spikes)
    #pylab.plot(numpy.cumsum(numpy.array(V_cpu>-0.05, dtype=int)))
    #pylab.show()
    #
    #exit()

    #print spikes[:102]
    #print gpu_cumblocksumcount.get()[-1]
    #print sum(V_cpu>-0.05)
    S1 = spikes[:gpu_cumblocksumcount.get()[-1]]
    S2 = (V_cpu > -0.05).nonzero()[0]
    S1.sort()
    S2.sort()
    #print S1
    #print S2
    print 'Passed spikes test:', (S1 == S2).all() # TODO: not quite there yet...

    exit()
else:
    threshold_blocksumcount.prepared_call(*threshold_blocksumcount_args)
    autoinit.context.synchronize()
    threshold_cumsum.prepared_call(*threshold_cumsum_args)
    autoinit.context.synchronize()
    threshold_compact.prepared_call(*threshold_compact_args)
    autoinit.context.synchronize()

def run_sim():
    if use_atomic_threshold:
        for t in xrange(duration):
            spike_index[0] = 0
            drv.memcpy_htod(gpu_spike_index, spike_index)
            threshold.launch_grid(*grid)
    else:
        for t in xrange(duration):
            threshold_blocksumcount.launch_grid(*grid)
            autoinit.context.synchronize()
            threshold_cumsum.launch_grid(*grid)
            autoinit.context.synchronize()
            threshold_compact.launch_grid(*grid)
            autoinit.context.synchronize()


def run_sim_cpu():
    for t in xrange(duration):
        Vt = -0.05
        code = """
                int numspikes=0;
                for(int i=0;i<N;i++)
                    if(V_cpu(i)>Vt)
                        spikes(numspikes++) = i;
                return_val = numspikes;
                """
        numspikes = weave.inline(code, ['spikes', 'V_cpu', 'Vt', 'N'],
                                 compiler='gcc',
                                 type_converters=weave.converters.blitz)

start = time.time()
run_sim()
timetaken_gpu = time.time() - start

start = time.time()
run_sim_cpu()
timetaken_cpu = time.time() - start

print 'N:', N, 'nspike', nspike, 'blocksize:', blocksize, 'numsteps:', duration
print 'Atomic threshold algorithm:', use_atomic_threshold
print 'GPU time:', timetaken_gpu
print 'CPU time:', timetaken_cpu



from base import *

__all__ = [
         'ConnectionVector', 'SparseConnectionVector', 'DenseConnectionVector',
         ]

class ConnectionVector(object):
    '''
    Base class for connection vectors, just used for defining the interface
    
    ConnectionVector objects are returned by ConnectionMatrix objects when
    they retrieve rows or columns. At the moment, there are two choices,
    sparse or dense.
    
    This class has no real function at the moment.
    '''
    def todense(self):
        return NotImplemented

    def tosparse(self):
        return NotImplemented


class DenseConnectionVector(ConnectionVector, numpy.ndarray):
    '''
    Just a numpy array.
    '''
    def __new__(subtype, arr):
        return numpy.array(arr, copy=False).view(subtype)

    def todense(self):
        return self

    def tosparse(self):
        return SparseConnectionVector(len(self), self.nonzero(), self)


class SparseConnectionVector(ConnectionVector, numpy.ndarray):
    '''
    Sparse vector class
    
    A sparse vector is typically a row or column of a sparse matrix. This
    class can be treated in many cases as if it were just a vector without
    worrying about the fact that it is sparse. For example, if you write
    ``2*v`` it will evaluate to a new sparse vector. There is one aspect
    of the semantics which is potentially confusing. In a binary operation
    with a dense vector such as ``sv+dv`` where ``sv`` is sparse and ``dv``
    is dense, the result will be a sparse vector with zeros where ``sv``
    has zeros, the potentially nonzero elements of ``dv`` where ``sv`` has
    no entry will be simply ignored. It is for this reason that it is a
    ``SparseConnectionVector`` and not a general ``SparseVector``, because
    these semantics make sense for rows and columns of connection matrices
    but not in general.
    
    Implementation details:
    
    The underlying numpy array contains the values, the attribute ``n`` is
    the length of the sparse vector, and ``ind`` is an array of the indices
    of the nonzero elements.
    '''
    def __new__(subtype, n, ind, data):
        x = numpy.array(data, copy=False).view(subtype)
        x.n = n
        x.ind = ind
        return x

    def __array_finalize__(self, orig):
        # the array is passed through this function after standard numpy operations,
        # this ensures that the indices are kept from the original array. This makes,
        # for example, sin(x) do the right thing for x a sparse vector.
        try:
            self.ind = orig.ind
            self.n = orig.n
        except AttributeError:
            pass
        return self

    def todense(self):
        x = zeros(self.n)
        x[self.ind] = self
        return x

    def tosparse(self):
        return self
    # This is a list of the binary operations that numpy arrays support.
    modifymeths = ['__add__', '__and__',
         '__div__', '__divmod__', '__eq__',
         '__floordiv__', '__ge__', '__gt__', '__iadd__', '__iand__', '__idiv__',
         '__ifloordiv__', '__ilshift__', '__imod__', '__imul__',
         '__ior__', '__ipow__', '__irshift__', '__isub__', '__itruediv__',
         '__ixor__', '__le__', '__lshift__', '__lt__', '__mod__', '__mul__',
         '__ne__', '__or__', '__pow__', '__radd__', '__rand__', '__rdiv__',
         '__rdivmod__', '__rfloordiv__', '__rlshift__',
         '__rmod__', '__rmul__', '__ror__', '__rpow__', '__rrshift__', '__rshift__',
         '__rsub__', '__rtruediv__', '__rxor__', '__sub__', '__truediv__', '__xor__']
    # This template function (where __add__ is replaced by any of the methods above) implements
    # the semantics described in this class' docstring when operating with a dense vector.
    template = '''
def __add__(self, other):
    if isinstance(other, SparseConnectionVector):
        # Note that removing this check is potentially dangerous, but only in weird circumstances would it cause
        # any problems, and leaving it in causes problems for STDP with DelayConnection (because the indices are
        # not the same, but should be presumed to be equal).
        #if other.ind is not self.ind:
        #    raise TypeError('__add__(SparseConnectionVector, SparseConnectionVector) only defined if indices are the same')
        return SparseConnectionVector(self.n, self.ind, numpy.ndarray.__add__(asarray(self), asarray(other)))
    if isinstance(other, numpy.ndarray):
        return SparseConnectionVector(self.n, self.ind, numpy.ndarray.__add__(asarray(self), other[self.ind]))
    return SparseConnectionVector(self.n, self.ind, numpy.ndarray.__add__(asarray(self), other))
'''.strip()
    # this substitutes any of the method names in the modifymeths list for __add__ in the template
    # above and then executes them, i.e. adding them as methods to the class. When the behaviour is
    # stable, this can be replaced by the explicit definitions but it may as well be left as it is for
    # the moment (it's slower at import time, but not at run time, and errors are more difficult to
    # catch when it is done like this).
    for m in modifymeths:
        s = template.replace('__add__', m)
        exec s
    del modifymeths, template
